@@ -15,13 +15,15 @@
 #include <stdarg.h>
 #include <link-grammar/api.h>
 
-/* The functions in this file do several things: () take a linkage
- involving fat links and expand it into a sequence of linkages
- (involving a subset of the given words), one for each way of
- eliminating the conjunctions.  () determine if a linkage involving
- fat links has a structural violation.  () make sure each of the expanded
- linkages has a consistent post-processing behavior.  () compute the
- cost of the linkage. */
+/**
+ * The functions in this file do several things: () take a linkage
+ * involving fat links and expand it into a sequence of linkages
+ * (involving a subset of the given words), one for each way of
+ * eliminating the conjunctions.  () determine if a linkage involving
+ * fat links has a structural violation.  () make sure each of the expanded
+ * linkages has a consistent post-processing behavior.  () compute the
+ * cost of the linkage.
+ */
 
 typedef struct patch_element_struct Patch_element;
 struct patch_element_struct
@@ -51,35 +53,39 @@ struct analyze_context_s
 	Patch_element patch_array[MAX_LINKS];
 };
 
-typedef struct DIS_node_struct DIS_node;
 typedef struct CON_node_struct CON_node;
 typedef struct CON_list_struct CON_list;
 typedef struct DIS_list_struct DIS_list;
 typedef struct Links_to_patch_struct Links_to_patch;
 
-struct DIS_node_struct {
+struct DIS_node_struct
+{
 	CON_list * cl;     /* the list of children */
 	List_o_links * lol;/* the links that comprise this region of the graph */
 	int word;          /* the word defining this node */
 };
 
-struct CON_node_struct {
+struct CON_node_struct
+{
 	DIS_list * dl;     /* the list of children */
 	DIS_list * current;/* defines the current child */
 	int word;          /* the word defining this node */
 };
 
-struct DIS_list_struct {
+struct DIS_list_struct
+{
 	DIS_list * next;
 	DIS_node * dn;
 };
 
-struct CON_list_struct {
+struct CON_list_struct
+{
 	CON_list * next;
 	CON_node * cn;
 };
 
-struct Links_to_patch_struct {
+struct Links_to_patch_struct
+{
 	Links_to_patch * next;
 	int link;
 	char dir;   /* this is 'r' or 'l' depending on which end of the link
@@ -149,7 +155,7 @@ static void copy_full_link(Link **dest, Link *src)
 /* end new code 9/97 ALB */
 
 
-/** 
+/**
  * Constructs a graph in the wordlinks array based on the contents of
  * the global link_array.  Makes the wordlinks array point to a list of
  * words neighboring each word (actually a list of links).  This is a
@@ -165,17 +171,19 @@ static void build_digraph(analyze_context_t *actx, Parse_info pi)
 	List_o_links * lol;
 	N_fat = 0;
 
-	for (i=0; i<pi->N_words; i++)
+	for (i = 0; i < pi->N_words; i++)
 	{
 		actx->word_links[i] = NULL;
 	}
-	for (link=0; link<pi->N_links; link++)
+
+	for (link = 0; link < pi->N_links; link++)
 	{
 		lp = &(pi->link_array[link]);
 		i = lp->lc->label;
 		if (i < NORMAL_LABEL) {   /* one of those special links for either-or, etc */
 			continue;
 		}
+
 		lol = (List_o_links *) xalloc(sizeof(List_o_links));
 		lol->next = actx->word_links[lp->l];
 		actx->word_links[lp->l] = lol;
@@ -189,6 +197,7 @@ static void build_digraph(analyze_context_t *actx, Parse_info pi)
 		} else {
 			lol->dir = -1;
 		}
+
 		lol = (List_o_links *) xalloc(sizeof(List_o_links));
 		lol->next = actx->word_links[lp->r];
 		actx->word_links[lp->r] = lol;
@@ -211,17 +220,16 @@ static void build_digraph(analyze_context_t *actx, Parse_info pi)
 static int is_CON_word(int w, List_o_links **wordlinks)
 {
 	List_o_links * lol;
-	for (lol = wordlinks[w]; lol!=NULL; lol = lol->next) {
-		if (lol->dir == 1) {
-			return TRUE;
-		}
+	for (lol = wordlinks[w]; lol != NULL; lol = lol->next)
+	{
+		if (lol->dir == 1) return TRUE;
 	}
 	return FALSE;
 }
 
 static DIS_node * build_DIS_node(analyze_context_t*, int);
 
-/** 
+/**
  * This word is a CON word (has fat links down).  Build the tree for it.
  */
 static CON_node * build_CON_node(analyze_context_t *actx, int w)
@@ -230,7 +238,7 @@ static CON_node * build_CON_node(analyze_context_t *actx, int w)
 	CON_node * a;
 	DIS_list * d, *dx;
 	d = NULL;
-	for (lol = actx->word_links[w]; lol!=NULL; lol = lol->next)
+	for (lol = actx->word_links[w]; lol != NULL; lol = lol->next)
 	{
 		if (lol->dir == 1)
 		{
@@ -281,9 +289,14 @@ static CON_list * c_dfs(analyze_context_t *actx,
 		}
 		else if (lol->dir == 0)
 		{
+			/* Make a copy of the link */
 			lolx = (List_o_links *) xalloc(sizeof(List_o_links));
-			lolx->next = start_dn->lol;
+			lolx->word = lol->word;
+			lolx->dir = lol->dir;
 			lolx->link = lol->link;
+
+			/* Chain it into place */
+			lolx->next = start_dn->lol;
 			start_dn->lol = lolx;
 			c = c_dfs(actx, lol->word, start_dn, c);
 		}
@@ -305,7 +318,7 @@ static CON_list * c_dfs(analyze_context_t *actx,
  * region reachable via thin links, and put all reachable nodes with fat
  * links out of them in its list of children.
  */
-static DIS_node * build_DIS_node(analyze_context_t *actx, 
+static DIS_node * build_DIS_node(analyze_context_t *actx,
                                  int w)
 {
 	DIS_node * dn;
@@ -330,6 +343,22 @@ static void height_dfs(analyze_context_t *actx, int w, int height)
 	}
 }
 
+#if defined(macintosh) || defined(__APPLE__) || defined(__APPLE_CC__)
+/* Assume this is OSX */
+/* Apple Mac, as a security precaution, uses a non-executable stack, 
+ * with a broken trampoline, and seg-faults on automatic functions.
+ * The alternative is to declare a static global, which, of course,
+ * is not thread-safe.
+ */
+static analyze_context_t *global_actx;
+static int comp_height(const void *va, const void *vb)
+{
+	const int *a = va;
+	const int *b = vb;
+	return global_actx->dfs_height[*b] - global_actx->dfs_height[*a];
+};
+#endif /* __APPLE__ */
+
 static DIS_node * build_DIS_CON_tree(analyze_context_t *actx, Parse_info pi)
 {
 	int xw, w;
@@ -337,6 +366,13 @@ static DIS_node * build_DIS_CON_tree(analyze_context_t *actx, Parse_info pi)
 	CON_list * child, * xchild;
 	List_o_links * lol, * xlol;
 
+#if defined(macintosh) || defined(__APPLE__) || defined(__APPLE_CC__)
+#ifdef USE_PTHREADS
+#error pthreads not supported on OSX until the use of this global is fixed!
+#else
+	global_actx = actx;
+#endif /* USE_PTHREADS */
+#else
 	/* Declare the compare function here, since it needs access to the
 	 * stack varaiable actx, which would otherwise need to be delcared
 	 * global (and thus not thread-safe).
@@ -344,20 +380,21 @@ static DIS_node * build_DIS_CON_tree(analyze_context_t *actx, Parse_info pi)
 	auto int comp_height(const void *, const void *);
 	int comp_height(const void *va, const void *vb)
 	{
-		const int *a = va; 
+		const int *a = va;
 		const int *b = vb;
 		return actx->dfs_height[*b] - actx->dfs_height[*a];
 	};
+#endif /* __APPLE__ */
 
 	/* The algorithm used here to build the DIS_CON tree depends on
-	   the search percolating down from the "top" of the tree.  The
-	   original version of this started its search at the wall.  This
-	   was fine because doing a DFS from the wall explore the tree in
-	   the right order.
-
-	   However, in order to handle null links correctly, a more careful
-	   ordering process must be used to explore the tree.  We use
-	   dfs_height[] for this.
+	 * the search percolating down from the "top" of the tree.  The
+	 * original version of this started its search at the wall.  This
+	 * was fine because doing a DFS from the wall explore the tree in
+	 * the right order.
+	 *
+	 * However, in order to handle null links correctly, a more careful
+	 * ordering process must be used to explore the tree.  We use
+	 * dfs_height[] for this.
 	 */
 
 	for (w=0; w < pi->N_words; w++) actx->dfs_height[w] = 0;
@@ -365,13 +402,13 @@ static DIS_node * build_DIS_CON_tree(analyze_context_t *actx, Parse_info pi)
 
 	for (w=0; w < pi->N_words; w++) actx->height_perm[w] = w;
 
-	qsort(actx->height_perm, pi->N_words, 
+	qsort(actx->height_perm, pi->N_words,
 		sizeof(actx->height_perm[0]), comp_height);
 
 	for (w=0; w<pi->N_words; w++) actx->dfs_root_word[w] = -1;
 
 	dnroot = NULL;
-	for (xw=0; xw < pi->N_words; xw++)
+	for (xw = 0; xw < pi->N_words; xw++)
 	{
 		w = actx->height_perm[xw];
 		if (actx->dfs_root_word[w] == -1)
@@ -389,7 +426,8 @@ static DIS_node * build_DIS_CON_tree(analyze_context_t *actx, Parse_info pi)
 					child->next = dnroot->cl;
 					dnroot->cl = child;
 				}
-				for (lol = dn->lol; lol != NULL; lol = xlol) {
+				for (lol = dn->lol; lol != NULL; lol = xlol)
+				{
 					xlol = lol->next;
 					lol->next = dnroot->lol;
 					dnroot->lol = lol;
@@ -404,35 +442,44 @@ static DIS_node * build_DIS_CON_tree(analyze_context_t *actx, Parse_info pi)
 static int advance_CON(CON_node *);
 
 /**
- * Cycically advance the current state of this DIS node.
- * If it's now at the beginning of its cycle" return FALSE;
- * Otherwise return TRUE;
+ * Cyclically advance the current state of this DIS node.
+ * If it's now at the beginning of its cycle, return FALSE;
+ * otherwise return TRUE. Together with the advance_CON()
+ * function, this can be used to iterate over the entire 
+ * DIS_CON tree.
  */
 static int advance_DIS(DIS_node * dn)
 {
 	CON_list * cl;
-	for (cl = dn->cl; cl!=NULL; cl=cl->next) {
-		if (advance_CON(cl->cn)) {
-			return TRUE;
-		}
+	for (cl = dn->cl; cl != NULL; cl = cl->next)
+	{
+		if (advance_CON(cl->cn)) return TRUE;
 	}
 	return FALSE;
 }
 
 /**
- * Cycically advance the current state of this CON node.
- * If it's now at the beginning of its cycle return FALSE;
- * Otherwise return TRUE;
+ * Cyclically advance the current state of this CON node.
+ * If it's now at the beginning of its cycle return FALSE,
+ * otherwise return TRUE. Together with the advance_CON()
+ * function, this can be used to iterate over the entire 
+ * DIS_CON tree.
  */
 static int advance_CON(CON_node * cn)
 {
-	if (advance_DIS(cn->current->dn)) {
+	if (advance_DIS(cn->current->dn))
+	{
 		return TRUE;
-	} else {
-		if (cn->current->next == NULL) {
+	}
+	else
+	{
+		if (cn->current->next == NULL)
+		{
 			cn->current = cn->dl;
 			return FALSE;
-		} else {
+		}
+		else
+		{
 			cn->current = cn->current->next;
 			return TRUE;
 		}
@@ -445,14 +492,14 @@ static void fill_patch_array_CON(analyze_context_t *, CON_node *, Links_to_patch
  * Patches up appropriate links in the patch_array for this DIS_node
  * and this patch list.
  */
-static void fill_patch_array_DIS(analyze_context_t *actx, 
+static void fill_patch_array_DIS(analyze_context_t *actx,
                                  DIS_node * dn, Links_to_patch * ltp)
 {
 	CON_list * cl;
 	List_o_links * lol;
 	Links_to_patch * ltpx;
 
-	for (lol = dn->lol; lol != NULL; lol=lol->next)
+	for (lol = dn->lol; lol != NULL; lol = lol->next)
 	{
 		actx->patch_array[lol->link].used = TRUE;
 	}
@@ -484,7 +531,7 @@ static void fill_patch_array_DIS(analyze_context_t *actx,
 	}
 }
 
-static void fill_patch_array_CON(analyze_context_t *actx, 
+static void fill_patch_array_CON(analyze_context_t *actx,
                                  CON_node * cn, Links_to_patch * ltp)
 {
 	List_o_links * lol;
@@ -524,15 +571,17 @@ static void free_digraph(analyze_context_t *actx, Parse_info pi)
 
 static void free_CON_tree(CON_node *);
 
-static void free_DIS_tree(DIS_node * dn)
+void free_DIS_tree(DIS_node * dn)
 {
 	List_o_links * lol, *lolx;
 	CON_list *cl, *clx;
-	for (lol=dn->lol; lol!=NULL; lol=lolx) {
+	for (lol = dn->lol; lol != NULL; lol = lolx)
+	{
 		lolx = lol->next;
 		xfree((void *) lol, sizeof(List_o_links));
 	}
-	for (cl = dn->cl; cl!=NULL; cl=clx) {
+	for (cl = dn->cl; cl != NULL; cl = clx)
+	{
 		clx = cl->next;
 		free_CON_tree(cl->cn);
 		xfree((void *) cl, sizeof(CON_list));
@@ -543,7 +592,8 @@ static void free_DIS_tree(DIS_node * dn)
 static void free_CON_tree(CON_node * cn)
 {
 	DIS_list *dl, *dlx;
-	for (dl = cn->dl; dl!=NULL; dl=dlx) {
+	for (dl = cn->dl; dl != NULL; dl = dlx)
+	{
 		dlx = dl->next;
 		free_DIS_tree(dl->dn);
 		xfree((void *) dl, sizeof(DIS_list));
@@ -601,7 +651,7 @@ static void and_dfs_commas(analyze_context_t *actx, Sentence sent, int w)
 	}
 }
 
-/** 
+/**
  * This function computes the "and cost", resulting from inequalities
  * in the length of and-list elements. It also computes other
  * information used to construct the "andlist" structure of linkage_info.
@@ -668,7 +718,7 @@ static Andlist * build_andlist(analyze_context_t *actx, Sentence sent)
 	return old_andlist;
 }
 
-/** 
+/**
  * This function defines the cost of a link as a function of its length.
  */
 static inline int cost_for_length(int length)
@@ -684,7 +734,8 @@ static int link_cost(Parse_info pi)
 {
 	int lcost, i;
 	lcost =  0;
-	for (i=0; i<pi->N_links; i++) {
+	for (i = 0; i < pi->N_links; i++)
+	{
 		lcost += cost_for_length(pi->link_array[i].r - pi->link_array[i].l);
 	}
 	return lcost;
@@ -701,21 +752,23 @@ static int unused_word_cost(Parse_info pi)
 {
 	int lcost, i;
 	lcost =  0;
-	for (i=0; i<pi->N_words; i++)
+	for (i = 0; i < pi->N_words; i++)
 		lcost += (pi->chosen_disjuncts[i] == NULL);
 	return lcost;
 }
 
-
+/** 
+ * Computes the cost of the current parse of the current sentence
+ * due to the cost of the chosen disjuncts.
+ */
 static int disjunct_cost(Parse_info pi)
 {
-/* computes the cost of the current parse of the current sentence     */
-/* due to the cost of the chosen disjuncts                            */
 	int lcost, i;
 	lcost =  0;
-	for (i=0; i<pi->N_words; i++) {
-	  if (pi->chosen_disjuncts[i] != NULL)
-		lcost += pi->chosen_disjuncts[i]->cost;
+	for (i = 0; i < pi->N_words; i++)
+	{
+		if (pi->chosen_disjuncts[i] != NULL)
+			lcost += pi->chosen_disjuncts[i]->cost;
 	}
 	return lcost;
 }
@@ -730,7 +783,8 @@ static int strictly_smaller_name(const char * s, const char * t)
 {
 	int strictness, ss, tt;
 	strictness = 0;
-	while ((*s!='\0') || (*t!='\0')) {
+	while ((*s!='\0') || (*t!='\0'))
+	{
 		if (*s == '\0') {
 			ss = '*';
 		} else {
@@ -762,7 +816,8 @@ static void compute_link_names(Sentence sent)
 	int i;
 	Parse_info pi = sent->parse_info;
 
-	for (i=0; i<pi->N_links; i++) {
+	for (i = 0; i < pi->N_links; i++)
+	{
 		pi->link_array[i].name = intersect_strings(sent,
 		   pi->link_array[i].lc->string, pi->link_array[i].rc->string);
 	}
@@ -784,19 +839,23 @@ static void compute_pp_link_names(Sentence sent, Sublinkage *sublinkage)
 	const char * s;
 	Parse_info pi = sent->parse_info;
 
-	for (i=0; i<pi->N_links; i++)
-	  {
+	for (i = 0; i < pi->N_links; i++)
+	{
 		if (sublinkage->link[i]->l == -1) continue;
 		if (!x_match(sent, sublinkage->link[i]->lc, sublinkage->link[i]->rc))
-		  replace_link_name(sublinkage->link[i], pi->link_array[i].name);
+		{
+			replace_link_name(sublinkage->link[i], pi->link_array[i].name);
+		}
 		else
-		  {
+		{
 			s = intersect_strings(sent, sublinkage->link[i]->lc->string,
 								  sublinkage->link[i]->rc->string);
+
 			if (strictly_smaller_name(s, pi->link_array[i].name))
-			  replace_link_name(sublinkage->link[i], pi->link_array[i].name);
-			else replace_link_name(sublinkage->link[i], s);
-		  }
+				replace_link_name(sublinkage->link[i], pi->link_array[i].name);
+			else
+				replace_link_name(sublinkage->link[i], s);
+		}
 	}
 }
 
@@ -850,7 +909,11 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 	li.improper_fat_linkage = actx->structure_violation;
 	li.inconsistent_domains = FALSE;
 	li.unused_word_cost = unused_word_cost(sent->parse_info);
+#ifdef USE_SAT_SOLVER
+	li.disjunct_cost = 0;
+#else
 	li.disjunct_cost = disjunct_cost(pi);
+#endif
 	li.null_cost = null_cost(pi);
 	li.link_cost = link_cost(pi);
 	li.and_cost = 0;
@@ -902,7 +965,9 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 			}
 		}
 
+#ifndef USE_SAT_SOLVER
 		compute_pp_link_array_connectors(sent, sublinkage);
+#endif
 		compute_pp_link_names(sent, sublinkage);
 
 		/* 'analyze_pass' logic added ALB 1/97 */
@@ -1007,7 +1072,11 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	li.unused_word_cost = unused_word_cost(sent->parse_info);
 	li.improper_fat_linkage = FALSE;
 	li.inconsistent_domains = FALSE;
+#ifdef USE_SAT_SOLVER
+	li.disjunct_cost = 0;
+#else
 	li.disjunct_cost = disjunct_cost(pi);
+#endif
 	li.null_cost = null_cost(pi);
 	li.link_cost = link_cost(pi);
 	li.andlist = NULL;
@@ -1048,6 +1117,68 @@ void extract_thin_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 	free_sublinkage(sublinkage);
 }
 
+#ifdef DBG
+static void prt_lol(Sentence sent , List_o_links *lol)
+{
+	/* It appears that the list of links is always even in length:
+	 * The head word first, followed by a modifier. 
+	 */
+	while (lol)
+	{
+		// printf ("%d ", lol->link);
+		printf ("%s ", sent->word[lol->word].string);
+		lol = lol->next;
+	}
+}
+
+static void prt_con_list(Sentence, CON_list *);
+static void prt_dis_list(Sentence sent, DIS_list *dis)
+{
+	while(dis)
+	{
+		/* There are three possibilities:
+ 		 * Either there's another conjunction (and we should print it)
+ 		 * Or there's a head word, with its modifiers in its list-o-links,
+ 		 * Or there's just the bare, naked word by itself.
+ 		 */
+		if (dis->dn->cl)
+		{
+			prt_con_list(sent, dis->dn->cl);
+		}
+		else if (dis->dn->lol)
+		{
+			printf("[");
+			prt_lol(sent, dis->dn->lol);
+			printf("]");
+		}
+		else 
+		{
+			int wd = dis->dn->word;
+			printf("%s ", sent->word[wd].string);
+		}
+		dis = dis->next;
+	}
+}
+
+static void prt_con_list(Sentence sent, CON_list *con)
+{
+	while(con)
+	{
+		int wd = con->cn->word;
+		printf("(%s ", sent->word[wd].string);
+		prt_dis_list(sent, con->cn->dl);
+		printf(") ");
+		con = con->next;
+	}
+}
+static void prt_dis_con_tree(Sentence sent, DIS_node *dis)
+{
+	prt_con_list(sent, dis->cl);
+	printf ("\n");
+}
+#else
+static inline void prt_dis_con_tree(Sentence sent, DIS_node *dis) {}
+#endif
 
 /**
  * This procedure mimics analyze_fat_linkage in order to
@@ -1114,7 +1245,7 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 	num_sublinkages = 0;
 	for (;;)
 	{
-		for (i=0; i<pi->N_links; i++)
+		for (i = 0; i < pi->N_links; i++)
 		{
 			actx->patch_array[i].used = actx->patch_array[i].changed = FALSE;
 			actx->patch_array[i].newl = pi->link_array[i].l;
@@ -1123,7 +1254,7 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 		}
 		fill_patch_array_DIS(actx, d_root, NULL);
 
-		for (i=0; i<pi->N_links; i++)
+		for (i = 0; i < pi->N_links; i++)
 		{
 			if (actx->patch_array[i].changed || actx->patch_array[i].used)
 			{
@@ -1137,12 +1268,15 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 			}
 		}
 
+#ifndef USE_SAT_SOLVER
 		compute_pp_link_array_connectors(sent, sublinkage);
 		compute_pp_link_names(sent, sublinkage);
+#endif
 
 		/* Don't copy the fat links into the linkage */
 		N_thin_links = 0;
-		for (i= 0; i<pi->N_links; ++i) {
+		for (i = 0; i < pi->N_links; ++i)
+		{
 			if (sublinkage->link[i]->l == -1) continue;
 			N_thin_links++;
 		}
@@ -1153,21 +1287,23 @@ void extract_fat_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 		linkage->sublinkage[num_sublinkages].pp_info = NULL;
 		linkage->sublinkage[num_sublinkages].violation = NULL;
 
-		for (i=0, j=0; i<pi->N_links; ++i) {
+		for (i = 0, j = 0; i < pi->N_links; ++i)
+		{
 			if (sublinkage->link[i]->l == -1) continue;
 			linkage->sublinkage[num_sublinkages].link[j++] =
 				excopy_link(sublinkage->link[i]);
 		}
 
-
 		num_sublinkages++;
 		if (!advance_DIS(d_root)) break;
 	}
 
 	free_sublinkage(sublinkage);
 	free_digraph(actx, pi);
-	free_DIS_tree(d_root);
+	if (linkage->dis_con_tree)
+		free_DIS_tree(linkage->dis_con_tree);
+	linkage->dis_con_tree = d_root;
+
+	prt_dis_con_tree(sent, d_root);
 }
 
-
-
