@@ -13,6 +13,9 @@
 
 #include <link-grammar/api.h>
 #include "preparation.h"
+#ifdef USE_SAT_SOLVER
+#include "sat-solver/sat-encoder.h"
+#endif
 
 /***************************************************************
 *
@@ -861,6 +864,9 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 
 	expression_prune(sent);
 	print_time(opts, "Finished expression pruning");
+#ifdef USE_SAT_SOLVER
+	sat_parse(sent, opts);
+#else
 	prepare_to_parse(sent, opts);
 
 	init_fast_matcher(sent);
@@ -889,6 +895,7 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 
 	free_table(sent);
 	free_fast_matcher(sent);
+#endif /* USE_SAT_SOLVER */
 	print_time(opts, "Finished parse");
 
 	return sent->num_valid_linkages;
@@ -918,18 +925,22 @@ Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
 	linkage->sent = sent;
 	linkage->opts = opts;
 	linkage->info = sent->link_info[k];
+	linkage->dis_con_tree = NULL;
 
 	extract_links(sent->link_info[k].index, sent->null_count, sent->parse_info);
 	compute_chosen_words(sent, linkage);
 
-	if (set_has_fat_down(sent)) {
+	if (set_has_fat_down(sent))
+	{
 		extract_fat_linkage(sent, opts, linkage);
 	}
-	else {
+	else
+	{
 		extract_thin_linkage(sent, opts, linkage);
 	}
 
-	if (sent->dict->postprocessor != NULL) {
+	if (sent->dict->postprocessor != NULL)
+	{
 	   linkage_post_process(linkage, sent->dict->postprocessor);
 	}
 
@@ -950,7 +961,8 @@ int linkage_set_current_sublinkage(Linkage linkage, int index)
 static void exfree_pp_info(PP_info *ppi)
 {
 	int i;
-	for (i=0; i<ppi->num_domains; ++i) {
+	for (i=0; i<ppi->num_domains; ++i)
+	{
 		exfree(ppi->domain_name[i], strlen(ppi->domain_name[i])+1);
 	}
 	if (ppi->num_domains > 0)
@@ -967,10 +979,12 @@ void linkage_delete(Linkage linkage)
 	/* Can happen on panic timeout or user error */
 	if (NULL == linkage) return;
 
-	for (i=0; i<linkage->num_words; ++i) {
+	for (i=0; i<linkage->num_words; ++i)
+	{
 		exfree((char *) linkage->word[i], strlen(linkage->word[i])+1);
 	}
 	exfree(linkage->word, sizeof(char *)*linkage->num_words);
+
 	for (i=0; i<linkage->num_sublinkages; ++i)
 	{
 		s = &(linkage->sublinkage[i]);
@@ -991,6 +1005,8 @@ void linkage_delete(Linkage linkage)
 		}
 	}
 	exfree(linkage->sublinkage, sizeof(Sublinkage)*linkage->num_sublinkages);
+	if (linkage->dis_con_tree)
+		free_DIS_tree(linkage->dis_con_tree);
 	exfree(linkage, sizeof(struct Linkage_s));
 }
 
