@@ -15,9 +15,10 @@
 
 #include <link-grammar/api.h>
 
-/*Temporary connectors used while converting expressions into disjunct lists */
+/* Temporary connectors used while converting expressions into disjunct lists */
 typedef struct Tconnector_struct Tconnector;
-struct Tconnector_struct{
+struct Tconnector_struct
+{
 	char multi;   /* TRUE if this is a multi-connector */
 	char dir;	 /* '-' for left and '+' for right */
 	Tconnector * next;
@@ -25,17 +26,20 @@ struct Tconnector_struct{
 };
 
 typedef struct clause_struct Clause;
-struct clause_struct {
+struct clause_struct
+{
 	Clause * next;
 	int cost;
 	int maxcost;
 	Tconnector * c;
 };
 
-static Tconnector * copy_Tconnectors(Tconnector * c) {
-/* This builds a new copy of the connector list pointed to by c.
-   Strings, as usual, are not copied.
-*/
+/**
+ * This builds a new copy of the connector list pointed to by c.
+ * Strings, as usual, are not copied.
+ */
+static Tconnector * copy_Tconnectors(Tconnector * c)
+{
 	Tconnector *c1;
 	if (c == NULL) return NULL;
 	c1 = (Tconnector *) xalloc(sizeof(Tconnector));
@@ -44,17 +48,21 @@ static Tconnector * copy_Tconnectors(Tconnector * c) {
 	return c1;
 }
 
-static void free_Tconnectors(Tconnector *e) {
+static void free_Tconnectors(Tconnector *e)
+{
 	Tconnector * n;
-	for(;e != NULL; e=n) {
+	for(;e != NULL; e=n)
+	{
 		n = e->next;
 		xfree((char *)e, sizeof(Tconnector));
 	}
 }
 
-static void free_clause_list(Clause *c) {
+static void free_clause_list(Clause *c)
+{
 	Clause *c1;
-	while (c != NULL) {
+	while (c != NULL)
+	{
 		c1 = c->next;
 		free_Tconnectors(c->c);
 		xfree((char *)c, sizeof(Clause));
@@ -77,8 +85,11 @@ static Clause * copy_clause(Clause * d) {
 }
 #endif /* UNUSED_FUNCTION */
 
-static Tconnector * Treverse(Tconnector *e) {
-/* reverse the order of the list e.  destructive */
+/**
+ * reverse the order of the list e.  destructive
+ */
+static Tconnector * Treverse(Tconnector *e)
+{
 	Tconnector * head, *x;
 	head = NULL;
 	while (e != NULL) {
@@ -90,8 +101,11 @@ static Tconnector * Treverse(Tconnector *e) {
 	return head;
 }
 
-static Connector * reverse(Connector *e) {
-/* reverse the order of the list e.  destructive */
+/**
+ * reverse the order of the list e.  destructive
+ */
+static Connector * reverse(Connector *e)
+{
 	Connector * head, *x;
 	head = NULL;
 	while (e != NULL) {
@@ -103,10 +117,12 @@ static Connector * reverse(Connector *e) {
 	return head;
 }
 
-static Tconnector * catenate(Tconnector * e1, Tconnector * e2) {
-/* Builds a new list of connectors that is the catenation of e1 with e2.
-   does not effect lists e1 or e2.   Order is maintained. */
-
+/**
+ * Builds a new list of connectors that is the catenation of e1 with e2.
+ * does not effect lists e1 or e2.   Order is maintained.
+ */
+static Tconnector * catenate(Tconnector * e1, Tconnector * e2)
+{
 	Tconnector * e, * head;
 	head = NULL;
 	for (;e1 != NULL; e1 = e1->next) {
@@ -124,8 +140,11 @@ static Tconnector * catenate(Tconnector * e1, Tconnector * e2) {
 	return Treverse(head);
 }
 
-static Tconnector * build_terminal(Exp * e) {
-	/* build the connector for the terminal node n */
+/**
+ * build the connector for the terminal node n
+ */
+static Tconnector * build_terminal(Exp * e)
+{
 	Tconnector * c;
 	c = (Tconnector *) xalloc(sizeof(Tconnector));
 	c->string = e->u.string;
@@ -135,7 +154,8 @@ static Tconnector * build_terminal(Exp * e) {
 	return c;
 }
 
-static int maxcost_of_expression(Exp *e) {
+static int maxcost_of_expression(Exp *e)
+{
 	E_list * e_list;
 	int m, m1;
 
@@ -151,15 +171,20 @@ static int maxcost_of_expression(Exp *e) {
 }
 
 #ifdef UNUSED_FUNCTION
-static int maxcost_of_sentence(Sentence sent) {
-/* This returns the maximum maxcost of any disjunct in the sentence */
-/* assumes the sentence expressions have been constructed */
+/**
+ * This returns the maximum maxcost of any disjunct in the sentence
+ * Assumes the sentence expressions have been constructed
+ */
+static int maxcost_of_sentence(Sentence sent)
+{
 	X_node * x;
 	int w, m, m1;
 	m = 0;
 
-	for (w=0; w<sent->length; w++) {
-		for (x=sent->word[w].x; x!=NULL; x = x->next){
+	for (w = 0; w < sent->length; w++)
+	{
+		for (x = sent->word[w].x; x != NULL; x = x->next)
+		{
 			m1 = maxcost_of_expression(x->exp),
 			m = MAX(m, m1);
 		}
@@ -169,23 +194,30 @@ static int maxcost_of_sentence(Sentence sent) {
 #endif /* UNUSED_FUNCTION */
 
 
-static Clause * build_clause(Exp *e, int cost_cutoff) {
-/* Build the clause for the expression e.  Does not change e */
-	Clause *c=NULL, *c1, *c2, *c3, *c4, *c_head;
+/**
+ * Build the clause for the expression e.  Does not change e
+ */
+static Clause * build_clause(Exp *e, int cost_cutoff)
+{
+	Clause *c = NULL, *c1, *c2, *c3, *c4, *c_head;
 	E_list * e_list;
 
 	assert(e != NULL, "build_clause called with null parameter");
-	if (e->type == AND_type) {
+	if (e->type == AND_type)
+	{
 		c1 = (Clause *) xalloc(sizeof (Clause));
 		c1->c = NULL;
 		c1->next = NULL;
 		c1->cost = 0;
 		c1->maxcost = 0 ;
-		for (e_list = e->u.l; e_list != NULL; e_list = e_list->next) {
+		for (e_list = e->u.l; e_list != NULL; e_list = e_list->next)
+		{
 			c2 = build_clause(e_list->e, cost_cutoff);
 			c_head = NULL;
-			for (c3 = c1; c3 != NULL; c3 = c3->next) {
-				for (c4 = c2; c4 != NULL; c4 = c4->next) {
+			for (c3 = c1; c3 != NULL; c3 = c3->next)
+			{
+				for (c4 = c2; c4 != NULL; c4 = c4->next)
+				{
 					c = (Clause *) xalloc(sizeof (Clause));
 					c->cost = c3->cost + c4->cost;
 					c->maxcost = MAX(c3->maxcost,c4->maxcost);
@@ -199,10 +231,13 @@ static Clause * build_clause(Exp *e, int cost_cutoff) {
 			c1 = c_head;
 		}
 		c = c1;
-	} else if (e->type == OR_type) {
+	}
+	else if (e->type == OR_type)
+	{
 		/* we'll catenate the lists of clauses */
 		c = NULL;
-		for (e_list = e->u.l; e_list != NULL; e_list = e_list->next) {
+		for (e_list = e->u.l; e_list != NULL; e_list = e_list->next)
+		{
 			c1 = build_clause(e_list->e, cost_cutoff);
 			while(c1 != NULL) {
 				c3 = c1->next;
@@ -211,30 +246,43 @@ static Clause * build_clause(Exp *e, int cost_cutoff) {
 				c1 = c3;
 			}
 		}
-	} else if (e->type == CONNECTOR_type) {
+	}
+	else if (e->type == CONNECTOR_type)
+	{
 		c = (Clause *) xalloc(sizeof(Clause));
 		c->c = build_terminal(e);
 		c->cost = 0;
 		c->maxcost = 0;
 		c->next = NULL;
-	} else {
+	}
+	else
+	{
 		assert(FALSE, "an expression node with no type");
 	}
 
 	/* c now points to the list of clauses */
-
-	for (c1=c; c1!=NULL; c1 = c1->next) {
+	for (c1 = c; c1 != NULL; c1 = c1->next)
+	{
 		c1->cost += e->cost;
-/*		c1->maxcost = MAX(c1->maxcost,e->cost);  */  /* <---- This is how Dennis had it.
-														I prefer the line below */
+		/*	c1->maxcost = MAX(c1->maxcost,e->cost);  */
+		/* Above is how Dennis had it. Someone changed it to below.
+		 * However, this can sometimes lead to a maxcost that is less
+		 * than the cost ! -- which seems wrong to me ... seems Dennis
+		 * had it right!?
+		 */
 		c1->maxcost += e->cost;
 	}
 	return c;
 }
 
-#ifdef UNUSED_FUNCTION
-static void print_connector_list(Connector * e) {
-	for (;e != NULL; e=e->next) {
+/* #define DEBUG */
+#ifdef DEBUG
+/* Misc printing functions, useful for debugging */
+
+static void print_connector_list(Connector * e)
+{
+	for (;e != NULL; e=e->next)
+	{
 		printf("%s",e->string);
 		if (e->label != NORMAL_LABEL) {
 			printf("%3d", e->label);
@@ -245,7 +293,8 @@ static void print_connector_list(Connector * e) {
 	}
 }
 
-static void print_Tconnector_list(Tconnector * e) {
+static void print_Tconnector_list(Tconnector * e)
+{
 	for (;e != NULL; e=e->next) {
 		if (e->multi) printf("@");
 		printf("%s",e->string);
@@ -254,19 +303,21 @@ static void print_Tconnector_list(Tconnector * e) {
 	}
 }
 
-static void print_clause_list(Clause * c) {
+static void print_clause_list(Clause * c)
+{
 	for (;c != NULL; c=c->next) {
 		printf("  Clause: ");
-		printf("(%2d, %2d)", c->cost, c->maxcost);
+		printf("(%2d, %2d) ", c->cost, c->maxcost);
 		print_Tconnector_list(c->c);
 		printf("\n");
 	}
 }
 
-static void print_disjunct_list(Disjunct * c) {
+static void print_disjunct_list(Disjunct * c)
+{
 	for (;c != NULL; c=c->next) {
 		printf("%10s: ", c->string);
-		printf("(%2d)", c->cost);
+		printf("(%d) ", c->cost);
 		print_connector_list(c->left);
 		printf(" <--> ");
 		print_connector_list(c->right);
@@ -284,8 +335,9 @@ static Connector * extract_connectors(Tconnector *e, int c)
 {
 	Connector *e1;
 	if (e == NULL) return NULL;
-	if (e->dir == c) {
-		e1 = init_connector((Connector *) xalloc(sizeof(Connector)));
+	if (e->dir == c)
+	{
+		e1 = connector_new();
 		e1->next = extract_connectors(e->next,c);
 		e1->multi = e->multi;
 		e1->string = e->string;
@@ -293,22 +345,26 @@ static Connector * extract_connectors(Tconnector *e, int c)
 		e1->priority = THIN_priority;
 		e1->word = 0;
 		return e1;
-	} else {
+	}
+	else
+	{
 		return extract_connectors(e->next,c);
 	}
 }
 
 /**
- * build a disjunct list out of the clause list c.
+ * Build a disjunct list out of the clause list c.
  * string is the print name of word that generated this disjunct.
  */
-static Disjunct * 
+static Disjunct *
 build_disjunct(Clause * cl, const char * string, int cost_cutoff)
 {
 	Disjunct *dis, *ndis;
 	dis = NULL;
-	for (;cl != NULL; cl=cl->next) {
-		if (cl->maxcost <= cost_cutoff) {
+	for (; cl != NULL; cl = cl->next)
+	{
+		if (cl->maxcost <= cost_cutoff)
+		{
 			ndis = (Disjunct *) xalloc(sizeof(Disjunct));
 			ndis->left = reverse(extract_connectors(cl->c, '-'));
 			ndis->right = reverse(extract_connectors(cl->c, '+'));
@@ -331,53 +387,26 @@ static Disjunct * build_disjuncts_for_X_node(X_node * x, int cost_cutoff)
 	return dis;
 }
 
+/**
+ * Build a list of disjuncts.
+ *
+ * This is mostly used only for counting the number of disjuncts
+ * (but is otherwise "almost" obsolete ??)
+ */
 Disjunct * build_disjuncts_for_dict_node(Dict_node *dn)
 {
-/* still need this for counting the number of disjuncts */
 	Clause *c ;
 	Disjunct * dis;
-/*				 print_expression(dn->exp);   */
-/*				 printf("\n");				*/
+	/*	 print_expression(dn->exp);   */
+	/*	 printf("\n");				*/
 	c = build_clause(dn->exp, NOCUTOFF);
-/*				 print_clause_list(c);		*/
+	/* print_clause_list(c); */
 	dis = build_disjunct(c, dn->string, NOCUTOFF);
+	/* print_disjunct_list(dis); */
 	free_clause_list(c);
 	return dis;
 }
 
-#if 0
-OBS
-OBS Disjunct * build_word_disjuncts(char * s) {
-OBS /* Looks up the word s in the dictionary.  Returns NULL if it's not there.
-OBS	If there, it builds the disjunct structure for the word, and returns
-OBS	a pointer to it.
-OBS */
-OBS	 Dict_node * dn;
-OBS	 Disjunct * dis;
-OBS
-OBS	 dn = dictionary_lookup(s);
-OBS
-OBS /*
-OBS	 x = dn;
-OBS	 printf("%s :", s);
-OBS	 while (dn != NULL) {
-OBS		printf("%s \n", dn->string);
-OBS		print_expression(dn->node);
-OBS		dn = dn->right;
-OBS	 }
-OBS	 dn = x;
-OBS */
-OBS	 dis = NULL;
-OBS
-OBS	 while (dn != NULL) {
-OBS		 dis = catenate_disjuncts(build_disjuncts_for_dict_node(dn), dis);
-OBS		 dn = dn->right;
-OBS	 }
-OBS				 /*	print_disjunct_list(dis); */
-OBS	 return dis;
-OBS }
-#endif
-
 /**
  * build_word_expressions() -- build list of expressions for a word
  *
@@ -394,7 +423,8 @@ X_node * build_word_expressions(Sentence sent, const char * s)
 	dn = dn_head;
 
 	x = NULL;
-	while (dn != NULL) {
+	while (dn != NULL)
+	{
 		y = (X_node *) xalloc(sizeof(X_node));
 		y->next = x;
 		x = y;
@@ -407,18 +437,21 @@ X_node * build_word_expressions(Sentence sent, const char * s)
 }
 
 /**
- * We've already built the sentence expressions.  This turns them into
- * disjuncts.
+ * Turn sentence expressions into disjuncts.
+ * Sentence expressions must have been built, before calling this routine.
  */
 void build_sentence_disjuncts(Sentence sent, int cost_cutoff)
 {
 	Disjunct * d;
 	X_node * x;
 	int w;
-	for (w=0; w<sent->length; w++) {
+	for (w = 0; w < sent->length; w++)
+	{
 		d = NULL;
-		for (x=sent->word[w].x; x!=NULL; x = x->next){
-			d = catenate_disjuncts(build_disjuncts_for_X_node(x, cost_cutoff),d);
+		for (x = sent->word[w].x; x != NULL; x = x->next)
+		{
+			Disjunct *dx = build_disjuncts_for_X_node(x, cost_cutoff);
+			d = catenate_disjuncts(dx, d);
 		}
 		sent->word[w].d = d;
 	}
