@@ -26,29 +26,36 @@ typedef enum {NONE, STYPE, PTYPE, QTYPE, QDTYPE} WType;
 
 typedef struct
 {
-  int left;
-  int right;
-  const char * type;
-  char domain_type;
-  const char * start_link;
-  int start_num;
-  int subl;
-  int canon;
-  int valid;
-  int aux;	
-  /* 0: it's an ordinary VP (or other type);
-	 1: it's an AUX, don't print it;
-	 2: it's an AUX, and print it */
+	int left;
+	int right;
+	const char * type;
+	char domain_type;
+	const char * start_link;
+	int start_num;
+	int subl;
+	int canon;
+	int valid;
+#ifdef AUX_CODE_IS_DEAD
+	/* The only code that actually sets aux to a non-zero value is code
+	 * followed by code that zets it to zero. -- its dead code, and so 
+	 * aux is never actually used. Comment this code out.
+	 */
+	int aux;	
+	/* 0: it's an ordinary VP (or other type);
+	 * 1: it's an AUX, don't print it;
+	 * 2: it's an AUX, and print it
+	 */
+#endif /* AUX_CODE_IS_DEAD */
 } constituent_t;
 
 /* XXX it seems like the old code worked fine with MAX_ELTS=10 */
 #define MAX_ELTS 100
 typedef struct
 {
-  int num;
-  int e[MAX_ELTS];
-  int valid;
-} andlist_t; 
+	int num;
+	int e[MAX_ELTS];
+	int valid;
+} andlist_t;
 
 /*
  * Context used to store assorted intermediate data
@@ -436,7 +443,7 @@ static int find_next_element(con_context_t *ctxt,
 			 * only from the names?
 			 */
 			prt_error("Error: Constituent overflowed andlist!\n");
-			return (num_lists-1); 
+			return (num_lists-1);
 		}
 	}
 	return num_lists;
@@ -467,10 +474,10 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 	   (the index number of the lowest-numbered constituent
 	   identical to it) */
 
-	for (c1=0; c1<numcon_total; c1++)
+	for (c1 = 0; c1 < numcon_total; c1++)
 	{
 		if (ctxt->constituent[c1].canon != c1) continue;
-		for (c2=c1+1; c2<numcon_total; c2++)
+		for (c2 = c1 + 1; c2 < numcon_total; c2++)
 		{
 			if ((ctxt->constituent[c1].left == ctxt->constituent[c2].left) &&
 				(ctxt->constituent[c1].right == ctxt->constituent[c2].right) &&
@@ -482,32 +489,32 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 	}
 
 	/* If constituents A and B in different sublinkages X and Y
-	   have one endpoint in common, but A is larger at the other end,
-	   and B has no duplicate in X, then declare B invalid. (Example:
-	   " [A [B We saw the cat B] and the dog A] " */
-
-	for (c1=0; c1<numcon_total; c1++)
+	 * have one endpoint in common, but A is larger at the other end,
+	 * and B has no duplicate in X, then declare B invalid. (Example:
+	 * " [A [B We saw the cat B] and the dog A] "
+	 */
+	for (c1 = 0; c1 < numcon_total; c1++)
 	{
 		if (ctxt->constituent[c1].valid == 0) continue;
-		for (c2=0; c2<numcon_total; c2++)
+		for (c2 = 0; c2 < numcon_total; c2++)
 		{
 			if (ctxt->constituent[c2].subl == ctxt->constituent[c1].subl) continue;
-			ok=1;
+			ok = 1;
 			/* Does c2 have a duplicate in the sublinkage containing c1?
 			   If so, bag it */
-			for (c3=0; c3<numcon_total; c3++)
+			for (c3 = 0; c3 < numcon_total; c3++)
 			{
 				if ((ctxt->constituent[c2].canon == ctxt->constituent[c3].canon) &&
 					(ctxt->constituent[c3].subl == ctxt->constituent[c1].subl))
-					ok=0;
+					ok = 0;
 			}
-			for (c3=0; c3<numcon_total; c3++)
+			for (c3 = 0; c3 < numcon_total; c3++)
 			{
 				if ((ctxt->constituent[c1].canon == ctxt->constituent[c3].canon) &&
 					(ctxt->constituent[c3].subl == ctxt->constituent[c2].subl))
-					ok=0;
+					ok = 0;
 			}
-			if (ok==0) continue;
+			if (ok == 0) continue;
 			if ((ctxt->constituent[c1].left == ctxt->constituent[c2].left) &&
 				(ctxt->constituent[c1].right > ctxt->constituent[c2].right) &&
 				(strcmp(ctxt->constituent[c1].type, ctxt->constituent[c2].type) == 0))
@@ -525,25 +532,25 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 	}
 
 	/* Now go through and find duplicates; if a pair is found,
-	   mark one as invalid. (It doesn't matter if they're in the
-	   same sublinkage or not) */
-
-	for (c1=0; c1<numcon_total; c1++)
+	 * mark one as invalid. (It doesn't matter if they're in the
+	 * same sublinkage or not)
+	 */
+	for (c1 = 0; c1 < numcon_total; c1++)
 	{
 		if (ctxt->constituent[c1].valid == 0) continue;
-		for (c2=c1+1; c2<numcon_total; c2++)
+		for (c2 = c1 + 1; c2 < numcon_total; c2++)
 		{
 			if (ctxt->constituent[c2].canon == ctxt->constituent[c1].canon)
-				ctxt->constituent[c2].valid=0;
+				ctxt->constituent[c2].valid = 0;
 		}
 	}
 
 	/* Now we generate the and-lists. An and-list is a set of mutually
-	   exclusive constituents. Each constituent in the list may not
-	   be present in the same sublinkage as any of the others. */
-
-	num_lists=0;
-	for (c1=0; c1<numcon_total; c1++)
+	 * exclusive constituents. Each constituent in the list may not
+	 * be present in the same sublinkage as any of the others.
+	 */
+	num_lists = 0;
+	for (c1 = 0; c1 < numcon_total; c1++)
 	{
 		if (ctxt->constituent[c1].valid == 0) continue;
 		num_elements = 1;
@@ -553,7 +560,7 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 							  num_elements, num_lists);
 	}
 
-	if (verbosity>=2)
+	if (verbosity >= 2)
 	{
 		printf("And-lists:\n");
 		for (n=0; n<num_lists; n++)
@@ -568,23 +575,23 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 	}
 
 	/* Now we prune out any andlists that are subsumed by other
-	   andlists--e.g. if andlist X contains constituents A and B,
-	   and Y contains A B and C, we throw out X */
-
-	for (n=0; n < num_lists; n++)
+	 * andlists--e.g. if andlist X contains constituents A and B,
+	 * and Y contains A B and C, we throw out X
+	 */
+	for (n = 0; n < num_lists; n++)
 	{
 		ctxt->andlist[n].valid = 1;
-		for (n2=0; n2 < num_lists; n2++)
+		for (n2 = 0; n2 < num_lists; n2++)
 		{
-			if (n2==n) continue;
+			if (n2 == n) continue;
 			if (ctxt->andlist[n2].num < ctxt->andlist[n].num)
 				continue;
 
 			listmatch = 1;
-			for (a=0; a < ctxt->andlist[n].num; a++)
+			for (a = 0; a < ctxt->andlist[n].num; a++)
 			{
 				match = 0;
-				for (a2=0; a2 < ctxt->andlist[n2].num; a2++)
+				for (a2 = 0; a2 < ctxt->andlist[n2].num; a2++)
 				{
 					if (ctxt->andlist[n2].e[a2] == ctxt->andlist[n].e[a])
 						match = 1;
@@ -597,43 +604,43 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 	}
 
 	/* If an element of an andlist contains an element of another
-	   andlist, it must contain the entire andlist. */
-
-	for (n=0; n<num_lists; n++)
+	 * andlist, it must contain the entire andlist.
+	 */
+	for (n = 0; n < num_lists; n++)
 	{
 		if (ctxt->andlist[n].valid == 0)
 			continue;
-		for (a=0; (a < ctxt->andlist[n].num) && (ctxt->andlist[n].valid); a++)
+		for (a = 0; (a < ctxt->andlist[n].num) && (ctxt->andlist[n].valid); a++)
 		{
-			for (n2=0; (n2 < num_lists) && (ctxt->andlist[n].valid); n2++)
+			for (n2 = 0; (n2 < num_lists) && (ctxt->andlist[n].valid); n2++)
 			{
-				if ((n2==n) || (ctxt->andlist[n2].valid==0))
+				if ((n2 == n) || (ctxt->andlist[n2].valid == 0))
 					continue;
-				for (a2=0; (a2 < ctxt->andlist[n2].num) && (ctxt->andlist[n].valid); a2++)
+				for (a2 = 0; (a2 < ctxt->andlist[n2].num) && (ctxt->andlist[n].valid); a2++)
 				{
 					c1 = ctxt->andlist[n].e[a];
 					c2 = ctxt->andlist[n2].e[a2];
-					if (c1==c2)
+					if (c1 == c2)
 						continue;
-					if (!((ctxt->constituent[c2].left<=ctxt->constituent[c1].left) &&
-						  (ctxt->constituent[c2].right>=ctxt->constituent[c1].right)))
+					if (!((ctxt->constituent[c2].left <= ctxt->constituent[c1].left) &&
+						  (ctxt->constituent[c2].right >= ctxt->constituent[c1].right)))
 						continue;
 					if (verbosity >= 2)
 						printf("Found that c%d in list %d is bigger " \
 							   "than c%d in list %d\n", c2, n2, c1, n);
-					ok=1;
+					ok = 1;
 
 					/* An element of n2 contains an element of n.
-					   Now, we check to see if that element of n2
-					   contains ALL the elements of n.
-					   If not, n is invalid. */
-
-					for (a3=0; a3 < ctxt->andlist[n].num; a3++)
+					 * Now, we check to see if that element of n2
+					 * contains ALL the elements of n.
+					 * If not, n is invalid.
+					 */
+					for (a3 = 0; a3 < ctxt->andlist[n].num; a3++)
 					{
 						c3 = ctxt->andlist[n].e[a3];
 						if ((ctxt->constituent[c2].left>ctxt->constituent[c3].left) ||
 							(ctxt->constituent[c2].right<ctxt->constituent[c3].right))
-							ok=0;
+							ok = 0;
 					}
 					if (ok != 0)
 						continue;
@@ -643,7 +650,8 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 						printf("Eliminating andlist, " \
 							   "n=%d, a=%d, n2=%d, a2=%d: ",
 							   n, a, n2, a2);
-						for (a3=0; a3 < ctxt->andlist[n].num; a3++) {
+						for (a3 = 0; a3 < ctxt->andlist[n].num; a3++)
+						{
 							printf("%d ", ctxt->andlist[n].e[a3]);
 						}
 						printf("\n");
@@ -653,7 +661,6 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 		}
 	}
 
-
 	if (verbosity >= 2)
 	{
 		printf("And-lists after pruning:\n");
@@ -669,43 +676,50 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 	}
 
 	c1 = numcon_total;
-	for (n=0; n<num_lists; n++) {
+	for (n = 0; n < num_lists; n++)
+	{
 		if (ctxt->andlist[n].valid == 0) continue;
-		leftend=256;
-		rightend=-1;
-		for (a=0; a < ctxt->andlist[n].num; a++) {
+		leftend = 256;
+		rightend = -1;
+		for (a = 0; a < ctxt->andlist[n].num; a++)
+		{
 			c2 = ctxt->andlist[n].e[a];
-			if (ctxt->constituent[c2].left<leftend) {
-				leftend=ctxt->constituent[c2].left;
+			if (ctxt->constituent[c2].left < leftend)
+			{
+				leftend = ctxt->constituent[c2].left;
 			}
-			if (ctxt->constituent[c2].right>rightend) {
+			if (ctxt->constituent[c2].right > rightend)
+			{
 				rightend=ctxt->constituent[c2].right;
 			}
 		}
 
-		ctxt->constituent[c1].left=leftend;
-		ctxt->constituent[c1].right=rightend;
+		ctxt->constituent[c1].left = leftend;
+		ctxt->constituent[c1].right = rightend;
 		ctxt->constituent[c1].type = ctxt->constituent[c2].type;
 		ctxt->constituent[c1].domain_type = 'x';
-		ctxt->constituent[c1].valid=1;
+		ctxt->constituent[c1].valid = 1;
 		ctxt->constituent[c1].start_link = ctxt->constituent[c2].start_link;  /* bogus */
 		ctxt->constituent[c1].start_num = ctxt->constituent[c2].start_num;	/* bogus */
 
+#ifdef AUX_CODE_IS_DEAD /* See comments above */
 		/* If a constituent within the andlist is an aux (aux==1),
-		   set aux for the whole-list constituent to 2, also set
-		   aux for the smaller constituent to 2, meaning they'll both
-		   be printed (as an "X"). (If aux is 2 for the smaller
-		   constituent going in, the same thing should be done,
-		   though I doubt this ever happens.) */
-
+		 * set aux for the whole-list constituent to 2, also set
+		 * aux for the smaller constituent to 2, meaning they'll both
+		 * be printed (as an "X"). (If aux is 2 for the smaller
+		 * constituent going in, the same thing should be done,
+		 * though I doubt this ever happens.)
+		 */
 		for (a = 0; a < ctxt->andlist[n].num; a++)
 		{
 			c2 = ctxt->andlist[n].e[a];
-			if ((ctxt->constituent[c2].aux==1) || (ctxt->constituent[c2].aux==2)) {
-				ctxt->constituent[c1].aux=2;
-				ctxt->constituent[c2].aux=2;
+			if ((ctxt->constituent[c2].aux == 1) || (ctxt->constituent[c2].aux == 2))
+			{
+				ctxt->constituent[c1].aux = 2;
+				ctxt->constituent[c2].aux = 2;
 			}
 		}
+#endif /* AUX_CODE_IS_DEAD */
 
 		if (verbosity >= 2)
 			printf("Adding constituent:\n");
@@ -770,11 +784,11 @@ static void generate_misc_word_info(con_context_t * ctxt, Linkage linkage)
 static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_total)
 {
 	int c, c2, global_leftend_found, adjustment_made,
-		global_rightend_found, lastword, newcon_total=0;
+		global_rightend_found, lastword, newcon_total = 0;
 	Sentence sent;
 	sent = linkage_get_sentence(linkage);
 
-	for (c=0; c<numcon_total; c++)
+	for (c = 0; c < numcon_total; c++)
 	{
 		/* In a paraphrase construction ("John ran, he said"),
 		   the paraphrasing clause doesn't get
@@ -828,19 +842,22 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 		   seems to apply to prenominal adjectives (of all kinds).
 		   However, it also applies to number expressions ("QP"). */
 
-		if ((post_process_match("A", ctxt->constituent[c].start_link)==1) ||
-			(ctxt->constituent[c].domain_type=='d') ||
-			(ctxt->constituent[c].domain_type=='h')) {
-			if (ctxt->constituent[c].right-ctxt->constituent[c].left==0) {
-				ctxt->constituent[c].valid=0;
+		if ((post_process_match("A", ctxt->constituent[c].start_link) == 1) ||
+			(ctxt->constituent[c].domain_type == 'd') ||
+			(ctxt->constituent[c].domain_type == 'h')) {
+			if (ctxt->constituent[c].right-ctxt->constituent[c].left == 0)
+			{
+				ctxt->constituent[c].valid = 0;
 			}
 		}
 
-		if ((ctxt->constituent[c].domain_type=='h') &&
-			(strcmp(linkage->word[ctxt->constituent[c].left-1], "$")==0)) {
+		if ((ctxt->constituent[c].domain_type == 'h') &&
+			(strcmp(linkage->word[ctxt->constituent[c].left - 1], "$") == 0))
+		{
 			ctxt->constituent[c].left--;
 		}
 
+#ifdef AUX_CODE_IS_DEAD /* See comments at top */
 		/* If a constituent has type VP and its aux value is 2,
 		   this means it's an aux that should be printed; change its
 		   type to "X". If its aux value is 1, set "valid" to 0. (This
@@ -854,6 +871,7 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 		{
 			ctxt->constituent[c].valid = 0;
 		}
+#endif /* AUX_CODE_IS_DEAD */
 	}
 
 	numcon_total = numcon_total + newcon_total;
@@ -861,31 +879,38 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 	/* If there's a global S constituent that includes everything
 	   except a final period or question mark, extend it by one word */
 
-	for (c=0; c<numcon_total; c++) {
-		if ((ctxt->constituent[c].right==(linkage->num_words)-3) &&
-			(ctxt->constituent[c].left==1) &&
-			(strcmp(ctxt->constituent[c].type, "S")==0) &&
-			(strcmp(sent->word[(linkage->num_words)-2].string, ".")==0))
+	for (c = 0; c < numcon_total; c++)
+	{
+		if ((ctxt->constituent[c].right == linkage->num_words -3) &&
+			(ctxt->constituent[c].left == 1) &&
+			(strcmp(ctxt->constituent[c].type, "S") == 0) &&
+			(strcmp(sent->word[linkage->num_words -2].string, ".") == 0))
 			ctxt->constituent[c].right++;
 	}
 
 	/* If there's no S boundary at the very left end of the sentence,
 	   or the very right end, create a new S spanning the entire sentence */
 
-	lastword=(linkage->num_words)-2;
+	lastword = linkage->num_words - 2;
 	global_leftend_found = 0;
 	global_rightend_found = 0;
-	for (c=0; c<numcon_total; c++) {
-		if ((ctxt->constituent[c].left==1) && (strcmp(ctxt->constituent[c].type, "S")==0) &&
-			(ctxt->constituent[c].valid==1))
-			global_leftend_found=1;
+	for (c = 0; c < numcon_total; c++)
+	{
+		if ((ctxt->constituent[c].left == 1) && (strcmp(ctxt->constituent[c].type, "S") == 0) &&
+			(ctxt->constituent[c].valid == 1))
+		{
+			global_leftend_found = 1;
+		}
 	}
-	for (c=0; c<numcon_total; c++) {
-		if ((ctxt->constituent[c].right>=lastword) &&
-			(strcmp(ctxt->constituent[c].type, "S")==0) && (ctxt->constituent[c].valid==1))
-			global_rightend_found=1;
+	for (c = 0; c < numcon_total; c++)
+	{
+		if ((ctxt->constituent[c].right >= lastword) &&
+			(strcmp(ctxt->constituent[c].type, "S") == 0) && (ctxt->constituent[c].valid == 1))
+		{
+			global_rightend_found = 1;
+		}
 	}
-	if ((global_leftend_found==0) || (global_rightend_found==0))
+	if ((global_leftend_found == 0) || (global_rightend_found == 0))
 	{
 		c = numcon_total;
 		ctxt->constituent[c].left = 1;
@@ -905,12 +930,12 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 	while (1)
 	{
 		adjustment_made=0;
-		for (c=0; c<numcon_total; c++)
+		for (c = 0; c < numcon_total; c++)
 		{
-			if(ctxt->constituent[c].valid==0) continue;
-			for (c2=0; c2<numcon_total; c2++)
+			if(ctxt->constituent[c].valid == 0) continue;
+			for (c2 = 0; c2 < numcon_total; c2++)
 			{
-				if(ctxt->constituent[c2].valid==0) continue;
+				if(ctxt->constituent[c2].valid == 0) continue;
 				if ((ctxt->constituent[c].left < ctxt->constituent[c2].left) &&
 					(ctxt->constituent[c].right < ctxt->constituent[c2].right) &&
 					(ctxt->constituent[c].right >= ctxt->constituent[c2].left))
@@ -924,12 +949,12 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 				}
 			}
 		}
-		if (adjustment_made==0) break;
+		if (adjustment_made == 0) break;
 	}
 	return numcon_total;
 }
 
-/** 
+/**
  * This function generates a table, word_used[i][w], showing
  * whether each word w is used in each sublinkage i; if so,
  * the value for that cell of the table is 1.
@@ -939,27 +964,27 @@ static void count_words_used(con_context_t *ctxt, Linkage linkage)
 	int i, w, link, num_subl;
 
 	num_subl = linkage->num_sublinkages;
-	if(linkage->unionized==1 && num_subl>1) num_subl--;
+	if(linkage->unionized == 1 && num_subl > 1) num_subl--;
 
 	if (verbosity >= 2)
 		printf("Number of sublinkages = %d\n", num_subl);
 
 	for (i=0; i<num_subl; i++)
 	{
-		for (w=0; w<linkage->num_words; w++) ctxt->word_used[i][w]=0;
-		linkage->current=i;
-		for (link=0; link<linkage_get_num_links(linkage); link++)
+		for (w = 0; w < linkage->num_words; w++) ctxt->word_used[i][w] = 0;
+		linkage->current = i;
+		for (link = 0; link < linkage_get_num_links(linkage); link++)
 		{
-			ctxt->word_used[i][linkage_get_link_lword(linkage, link)]=1;
-			ctxt->word_used[i][linkage_get_link_rword(linkage, link)]=1;
+			ctxt->word_used[i][linkage_get_link_lword(linkage, link)] = 1;
+			ctxt->word_used[i][linkage_get_link_rword(linkage, link)] = 1;
 		}
 		if (verbosity >= 2)
 		{
 			printf("Sublinkage %d: ", i);
-			for (w=0; w<linkage->num_words; w++)
+			for (w = 0; w < linkage->num_words; w++)
 			{
-				if (ctxt->word_used[i][w]==0) printf("0 ");
-				if (ctxt->word_used[i][w]==1) printf("1 ");
+				if (ctxt->word_used[i][w] == 0) printf("0 ");
+				if (ctxt->word_used[i][w] == 1) printf("1 ");
 			}
 			printf("\n");
 		}
@@ -1036,7 +1061,7 @@ static const char * cons_of_domain(char domain_type)
 static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
                                           int numcon_total, int s)
 {
-	int d, c, leftlimit, l, leftmost, rightmost, w, c2, numcon_subl=0, w2;
+	int d, c, leftlimit, l, leftmost, rightmost, w, c2, numcon_subl = 0, w2;
 	List_o_links * dlink;
 	int rootright, rootleft, adjustment_made;
 	Sublinkage * subl;
@@ -1047,7 +1072,8 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 
 	subl = &linkage->sublinkage[s];
 
-	for (d=0, c=numcon_total; d<subl->pp_data.N_domains; d++, c++) {
+	for (d = 0, c = numcon_total; d < subl->pp_data.N_domains; d++, c++)
+	{
 		domain = subl->pp_data.domain_array[d];
 		rootright = linkage_get_link_rword(linkage, domain.start_link);
 		rootleft =  linkage_get_link_lword(linkage, domain.start_link);
@@ -1058,87 +1084,105 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 			(domain.type=='f') ||
 			(domain.type=='g') ||
 			(domain.type=='u') ||
-			(domain.type=='y')) {
+			(domain.type=='y'))
+		{
 			leftlimit = 0;
 			leftmost = linkage_get_link_lword(linkage, domain.start_link);
 			rightmost = linkage_get_link_lword(linkage, domain.start_link);
 		}
-		else {
-			leftlimit = linkage_get_link_lword(linkage, domain.start_link)+1;
+		else
+		{
+			leftlimit = linkage_get_link_lword(linkage, domain.start_link) + 1;
 			leftmost = linkage_get_link_rword(linkage, domain.start_link);
 			rightmost = linkage_get_link_rword(linkage, domain.start_link);
 		}
 
 		/* Start by assigning both left and right limits to the
-		   right word of the start link. This will always be contained
-		   in the constituent. This will also handle the case
-		   where the domain contains no links. */
-
-		for (dlink = domain.lol; dlink!=NULL; dlink=dlink->next) {
-			l=dlink->link;
+		 * right word of the start link. This will always be contained
+		 * in the constituent. This will also handle the case
+		 * where the domain contains no links.
+		 */
+		for (dlink = domain.lol; dlink != NULL; dlink = dlink->next)
+		{
+			l = dlink->link;
 
 			if ((linkage_get_link_lword(linkage, l) < leftmost) &&
 				(linkage_get_link_lword(linkage, l) >= leftlimit))
+			{
 				leftmost = linkage_get_link_lword(linkage, l);
+			}
 
 			if (linkage_get_link_rword(linkage, l) > rightmost)
+			{
 				rightmost = linkage_get_link_rword(linkage, l);
+			}
 		}
 
 		c--;
 		c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost,
 						cons_of_domain(domain.type));
 
-		if (domain.type=='z') {
+		if (domain.type == 'z')
+		{
 			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "S");
 		}
-		if (domain.type=='c') {
+		if (domain.type=='c')
+		{
 			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "S");
 		}
 		if ((post_process_match("Ce*", ctxt->constituent[c].start_link)==1) ||
-			(post_process_match("Rn", ctxt->constituent[c].start_link)==1)) {
+			(post_process_match("Rn", ctxt->constituent[c].start_link)==1))
+		{
 			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "SBAR");
 		}
 		if ((post_process_match("R*", ctxt->constituent[c].start_link)==1) ||
-			(post_process_match("MX#r", ctxt->constituent[c].start_link)==1)) {
-			w=leftmost;
-			if (strcmp(linkage->word[w], ",")==0) w++;
+			(post_process_match("MX#r", ctxt->constituent[c].start_link)==1))
+		{
+			w = leftmost;
+			if (strcmp(linkage->word[w], ",") == 0) w++;
 			c = add_constituent(ctxt, c, linkage, domain, w, w, "WHNP");
 		}
-		if (post_process_match("Mj", ctxt->constituent[c].start_link)==1) {
-			w=leftmost;
-			if (strcmp(linkage->word[w], ",")==0) w++;
+		if (post_process_match("Mj", ctxt->constituent[c].start_link) == 1)
+		{
+			w = leftmost;
+			if (strcmp(linkage->word[w], ",") == 0) w++;
 			c = add_constituent(ctxt, c, linkage, domain, w, w+1, "WHPP");
 			c = add_constituent(ctxt, c, linkage, domain, w+1, w+1, "WHNP");
 		}
 		if ((post_process_match("Ss#d", ctxt->constituent[c].start_link)==1) ||
-			(post_process_match("B#d", ctxt->constituent[c].start_link)==1)) {
+			(post_process_match("B#d", ctxt->constituent[c].start_link)==1))
+		{
 			c = add_constituent(ctxt, c, linkage, domain, rootleft, rootleft, "WHNP");
 			c = add_constituent(ctxt, c, linkage, domain,
 							rootleft, ctxt->constituent[c-1].right, "SBAR");
 		}
-		if (post_process_match("CP", ctxt->constituent[c].start_link)==1) {
-			if (strcmp(linkage->word[leftmost], ",")==0)
+		if (post_process_match("CP", ctxt->constituent[c].start_link)==1)
+		{
+			if (strcmp(linkage->word[leftmost], ",") == 0)
 				ctxt->constituent[c].left++;
 			c = add_constituent(ctxt, c, linkage, domain, 1, linkage->num_words-1, "S");
 		}
 		if ((post_process_match("MVs", ctxt->constituent[c].start_link)==1) ||
-			(domain.type=='f')) {
-			w=ctxt->constituent[c].left;
-			if (strcmp(linkage->word[w], ",")==0)
+			(domain.type=='f'))
+		{
+			w = ctxt->constituent[c].left;
+			if (strcmp(linkage->word[w], ",") == 0)
 				w++;
-			if (strcmp(linkage->word[w], "when")==0) {
+			if (strcmp(linkage->word[w], "when") == 0)
+			{
 				c = add_constituent(ctxt, c, linkage, domain, w, w, "WHADVP");
 			}
 		}
-		if (domain.type=='t') {
+		if (domain.type=='t')
+		{
 			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "S");
 		}
-		if ((post_process_match("QI", ctxt->constituent[c].start_link)==1) ||
-			(post_process_match("Mr", ctxt->constituent[c].start_link)==1) ||
-			(post_process_match("MX#d", ctxt->constituent[c].start_link)==1)) {
+		if ((post_process_match("QI", ctxt->constituent[c].start_link) == 1) ||
+			(post_process_match("Mr", ctxt->constituent[c].start_link) == 1) ||
+			(post_process_match("MX#d", ctxt->constituent[c].start_link) == 1))
+		{
 			w = leftmost;
-			if (strcmp(linkage->word[w], ",")==0) w++;
+			if (strcmp(linkage->word[w], ",") == 0) w++;
 			if (ctxt->wordtype[w] == NONE)
 				name = "WHADVP";
 			else if (ctxt->wordtype[w] == QTYPE)
@@ -1149,7 +1193,8 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 				assert(0, "Unexpected word type");
 			c = add_constituent(ctxt, c, linkage, domain, w, w, name);
 
-			if (ctxt->wordtype[w] == QDTYPE) {
+			if (ctxt->wordtype[w] == QDTYPE)
+			{
 				/* Now find the finite verb to the right, start an S */
 				/* Limit w2 to sentence length. */
 				// for( w2=w+1; w2 < ctxt->r_limit-1; w2++ )
@@ -1157,15 +1202,17 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 				  if ((ctxt->wordtype[w2] == STYPE) || (ctxt->wordtype[w2] == PTYPE)) break;
 
 				/* Adjust the right boundary of previous constituent */
-				ctxt->constituent[c].right = w2-1;
+				ctxt->constituent[c].right = w2 - 1;
 				c = add_constituent(ctxt, c, linkage, domain, w2, rightmost, "S");
-			  }
+			}
 		}
 
-		if (ctxt->constituent[c].domain_type=='\0') {
+		if (ctxt->constituent[c].domain_type == '\0')
+		{
 			prt_error("Error: no domain type assigned to constituent\n");
 		}
-		if (ctxt->constituent[c].start_link==NULL) {
+		if (ctxt->constituent[c].start_link == NULL)
+		{
 			prt_error("Error: no type assigned to constituent\n");
 		}
 	}
@@ -1220,9 +1267,11 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "NP", "SINV", "VP", 7);
 
 	adjust_subordinate_clauses(ctxt, linkage, numcon_total, numcon_subl);
-	for (c=numcon_total; c<numcon_total + numcon_subl; c++) {
+	for (c = numcon_total; c < numcon_total + numcon_subl; c++)
+	{
 		if ((ctxt->constituent[c].domain_type=='p') &&
-			(strcmp(linkage->word[ctxt->constituent[c].left], ",")==0)) {
+			(strcmp(linkage->word[ctxt->constituent[c].left], ",")==0))
+		{
 			ctxt->constituent[c].left++;
 		}
 	}
@@ -1232,34 +1281,40 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 	 * boundary, shift that boundary rightwards to the left boundary
 	 * of the other one.
 	 */
-	while (1) {
-		adjustment_made=0;
-		for (c=numcon_total; c<numcon_total + numcon_subl; c++) {
-			for (c2=numcon_total; c2<numcon_total + numcon_subl; c2++) {
+	while (1)
+	{
+		adjustment_made = 0;
+		for (c = numcon_total; c < numcon_total + numcon_subl; c++)
+		{
+			for (c2 = numcon_total; c2 < numcon_total + numcon_subl; c2++)
+			{
 				if ((ctxt->constituent[c].left < ctxt->constituent[c2].left) &&
 					(ctxt->constituent[c].right < ctxt->constituent[c2].right) &&
-					(ctxt->constituent[c].right >= ctxt->constituent[c2].left)) {
-
+					(ctxt->constituent[c].right >= ctxt->constituent[c2].left))
+				{
 					/* We've found two overlapping constituents.
 					   If one is larger, except the smaller one
 					   includes an extra comma, adjust the smaller one
 					   to exclude the comma */
 
-					if ((strcmp(linkage->word[ctxt->constituent[c2].right], ",")==0) ||
+					if ((strcmp(linkage->word[ctxt->constituent[c2].right], ",") == 0) ||
 						(strcmp(linkage->word[ctxt->constituent[c2].right],
-								"RIGHT-WALL")==0)) {
+								"RIGHT-WALL") == 0))
+					{
 						if (verbosity >= 2)
 							printf("Adjusting %d to fix comma overlap\n", c2);
 						adjust_for_right_comma(ctxt, linkage, c2);
-						adjustment_made=1;
+						adjustment_made = 1;
 					}
-					else if (strcmp(linkage->word[ctxt->constituent[c].left], ",")==0) {
+					else if (strcmp(linkage->word[ctxt->constituent[c].left], ",") == 0)
+					{
 						if (verbosity >= 2)
 							printf("Adjusting c %d to fix comma overlap\n", c);
 						adjust_for_left_comma(ctxt, linkage, c);
-						adjustment_made=1;
+						adjustment_made = 1;
 					}
-					else {
+					else
+					{
 						if (verbosity >= 2)
 						{
 							prt_error("Warning: the constituents aren't nested! "
@@ -1270,34 +1325,51 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 				}
 			}
 		}
-		if (adjustment_made==0) break;
+		if (adjustment_made == 0) break;
 	}
 
+#ifdef AUX_CODE_IS_DEAD 
+/* The code here is ifdef-dead as it appears to be dead, as the computation it does
+ * is immediately undone in the very next block.
+ */
 	/* This labels certain words as auxiliaries (such as forms of "be"
-	   with passives, forms of "have" wth past participles,
-	   "to" with infinitives). These words start VP's which include
-	   them. In Treebank I, these don't get printed unless they're part of an
-	   andlist, in which case they get labeled "X". (this is why we need to
-	   label them as "aux".) In Treebank II, however, they seem to be treated
-	   just like other verbs, so the "aux" stuff isn't needed. */
-
-
-	for (c=numcon_total; c<numcon_total + numcon_subl; c++) {
+	 * with passives, forms of "have" wth past participles,
+	 * "to" with infinitives). These words start VP's which include
+	 * them. In Treebank I, these don't get printed unless they're part of an
+	 * andlist, in which case they get labeled "X". (this is why we need to
+	 * label them as "aux".) In Treebank II, however, they seem to be treated
+	 * just like other verbs, so the "aux" stuff isn't needed.
+	 */
+	for (c = numcon_total; c < numcon_total + numcon_subl; c++)
+	{
 		ctxt->constituent[c].subl = linkage->current;
 		if (((ctxt->constituent[c].domain_type == 'v') &&
 			(ctxt->wordtype[linkage_get_link_rword(linkage,
-											 ctxt->constituent[c].start_num)]==PTYPE))
+											 ctxt->constituent[c].start_num)] == PTYPE))
 		   ||
 		   ((ctxt->constituent[c].domain_type == 't') &&
-			(strcmp(ctxt->constituent[c].type, "VP")==0))) {
-			ctxt->constituent[c].aux=1;
+			(strcmp(ctxt->constituent[c].type, "VP") == 0)))
+		{
+			ctxt->constituent[c].aux = 1;
+		}
+		else
+		{
+			ctxt->constituent[c].aux = 0;
 		}
-		else ctxt->constituent[c].aux=0;
 	}
+#endif /* AUX_CODE_IS_DEAD */
 
-	for (c=numcon_total; c<numcon_total + numcon_subl; c++) {
+	if (MAXCONSTITUENTS <= numcon_total + numcon_subl)
+	{
+		prt_error("Error: Too many constituents (a2).\n");
+		numcon_total = MAXCONSTITUENTS - numcon_subl;
+	}
+	for (c = numcon_total; c < numcon_total + numcon_subl; c++)
+	{
 		ctxt->constituent[c].subl = linkage->current;
-		ctxt->constituent[c].aux=0;
+#ifdef AUX_CODE_IS_DEAD /* See comments at top */
+		ctxt->constituent[c].aux = 0;
+#endif /* AUX_CODE_IS_DEAD */
 	}
 
 	return numcon_subl;
@@ -1319,14 +1391,14 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 
 	for (c = 0; c < numcon_total; c++)
 	{
-		leftdone[c]=0;
-		rightdone[c]=0;
+		leftdone[c] = 0;
+		rightdone[c] = 0;
 	}
 
 	if (verbosity >= 2)
 		printf("\n");			
 
-	for (w=1; w<linkage->num_words; w++)
+	for (w = 1; w < linkage->num_words; w++)
 	{
 		/* Skip left wall; don't skip right wall, since it may
 		   have constituent boundaries */
@@ -1335,7 +1407,7 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 		{
 			best = -1;
 			bestright = -1;
-			for (c=0; c < numcon_total; c++)
+			for (c = 0; c < numcon_total; c++)
 			{
 				if ((ctxt->constituent[c].left == w) &&
 					(leftdone[c] == 0) && (ctxt->constituent[c].valid == 1) &&
@@ -1348,10 +1420,12 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 				break;
 
 			leftdone[best] = 1;
-			/* have_open is a hack to avoid printing anything until 
+			/* have_open is a hack to avoid printing anything until
 			 * bracket is opened */
-			if (w == 1) have_opened = 0; 
+			if (w == 1) have_opened = 0;
+#ifdef AUX_CODE_IS_DEAD /* See comments at top */
 			if (ctxt->constituent[best].aux == 1) continue;
+#endif /* AUX_CODE_IS_DEAD */
 			have_opened = 1;
 			append_string(cs, "%c%s ", OPEN_BRACKET, ctxt->constituent[best].type);
 		}
@@ -1363,7 +1437,7 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 			strcpy(s, sent->word[w].string);
 
 			/* Constituent processing will crash if the sentence contains
-			 * square brackets, so we have to do something ... replace 
+			 * square brackets, so we have to do something ... replace
 			 * them with curly braces ... will have to do.
 			 */
 			p = strchr(s, OPEN_BRACKET);
@@ -1391,19 +1465,22 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 		{
 			best = -1;
 			bestleft = -1;
-			for(c=0; c<numcon_total; c++) {
-				if ((ctxt->constituent[c].right==w) &&
-					(rightdone[c]==0) && (ctxt->constituent[c].valid==1) &&
+			for(c = 0; c < numcon_total; c++)
+			{
+				if ((ctxt->constituent[c].right == w) &&
+					(rightdone[c] == 0) && (ctxt->constituent[c].valid == 1) &&
 					(ctxt->constituent[c].left > bestleft)) {
 					best = c;
 					bestleft = ctxt->constituent[c].left;
 				}
 			}
-			if (best==-1)
+			if (best == -1)
 				break;
-			rightdone[best]=1;
-			if (ctxt->constituent[best].aux==1)
+			rightdone[best] = 1;
+#ifdef AUX_CODE_IS_DEAD /* See comments at top */
+			if (ctxt->constituent[best].aux == 1)
 				continue;
+#endif /* AUX_CODE_IS_DEAD */
 			append_string(cs, "%s%c ", ctxt->constituent[best].type, CLOSE_BRACKET);
 		}
 	}
@@ -1436,8 +1513,9 @@ static char * do_print_flat_constituents(con_context_t *ctxt, Linkage linkage)
 	  if (verbosity >= 2)
 		printf("Number of sublinkages exceeds maximum: only considering first %d sublinkages\n", MAXSUBL);
 	}
-	if (linkage->unionized==1 && num_subl>1) num_subl--;
-	for (s=0; s<num_subl; s++)
+
+	if (linkage->unionized == 1 && num_subl > 1) num_subl--;
+	for (s = 0; s < num_subl; s++)
 	{
 		linkage_set_current_sublinkage(linkage, s);
 		linkage_post_process(linkage, pp);
@@ -1472,15 +1550,25 @@ static char * do_print_flat_constituents(con_context_t *ctxt, Linkage linkage)
 
 static char * print_flat_constituents(Linkage linkage)
 {
-	con_context_t ctxt;
-	return do_print_flat_constituents(&ctxt, linkage);
+	/* In principle, the ctxt could be allocated on stack, instead of
+	 * with malloc(). However, The java6 jvm (and MS Windows jvm's)
+	 * gives JNI clients only a small amount of stack space. Alloc'ing
+	 * this (rather large) structure  on stack will blow up the JVM.
+	 * This was discovered only after much work. Bummer.
+	 */
+	char * p;
+	con_context_t *ctxt = (con_context_t *) malloc (sizeof(con_context_t));
+	bzero(ctxt, sizeof(con_context_t));
+	p = do_print_flat_constituents(ctxt, linkage);
+	free(ctxt);
+	return p;
 }
 
 static CType token_type (char *token)
 {
-	if ((token[0] == OPEN_BRACKET) && (strlen(token)>1))
+	if ((token[0] == OPEN_BRACKET) && (strlen(token) > 1))
 		return OPEN_TOK;
-	if ((strlen(token)>1) && (token[strlen(token)-1] == CLOSE_BRACKET))
+	if ((strlen(token) > 1) && (token[strlen(token) - 1] == CLOSE_BRACKET))
 		return CLOSE_TOK;
 	return WORD_TOK;
 }
@@ -1551,7 +1639,7 @@ static void print_tree(String * cs, int indent, CNode * n, int o1, int o2)
 		{
 			char * p;
 			/* If the original string has left or right parens in it,
-			 * the printed string will be messed up by these ... 
+			 * the printed string will be messed up by these ...
 			 * so replace them by curly braces. What else can one do?
 			 */
 			p = strchr(m->label, '(');
@@ -1688,26 +1776,26 @@ void linkage_free_constituent_tree_str(char * s)
 
 char * linkage_constituent_node_get_label(const CNode *n)
 {
-  return n->label;
+	return n->label;
 }
 
 
 CNode * linkage_constituent_node_get_child(const CNode *n)
 {
-  return n->child;
+	return n->child;
 }
 
 CNode * linkage_constituent_node_get_next(const CNode *n)
 {
-  return n->next;
+	return n->next;
 }
 
 int linkage_constituent_node_get_start(const CNode *n)
 {
-  return n->start;
+	return n->start;
 }
 
 int linkage_constituent_node_get_end(const CNode *n)
 {
-  return n->end;
+	return n->end;
 }
