@@ -13,28 +13,27 @@
 
 #include <link-grammar/api.h>
 
-/* The first thing we do is we build a data structure to represent the
-   result of the entire parse search.  There will be a set of nodes
-   built for each call to the count() function that returned a non-zero
-   value, AND which is part of a valid linkage.  Each of these nodes
-   represents a valid continuation, and contains pointers to two other
-   sets (one for the left continuation and one for the righ
-   continuation).
-
+/**
+ * The first thing we do is we build a data structure to represent the
+ * result of the entire parse search.  There will be a set of nodes
+ * built for each call to the count() function that returned a non-zero
+ * value, AND which is part of a valid linkage.  Each of these nodes
+ * represents a valid continuation, and contains pointers to two other
+ * sets (one for the left continuation and one for the right
+ * continuation).
  */
 
-static Word * local_sent;
-static int	islands_ok;
-
-static Parse_set * dummy_set(void) {
+static Parse_set * dummy_set(void)
+{
 	static Parse_set ds;
 	ds.first = ds.current = NULL;
 	ds.count = 1;
 	return &ds;
 }
 
-static Parse_set * empty_set(void) {
-	/* returns an empty set of parses */
+/** Returns an empty set of parses */
+static Parse_set * empty_set(void)
+{
 	Parse_set *s;
 	s = (Parse_set *) xalloc(sizeof(Parse_set));
 	s->first = s->current = NULL;
@@ -42,7 +41,8 @@ static Parse_set * empty_set(void) {
 	return s;
 }
 
-static void free_set(Parse_set *s) {
+static void free_set(Parse_set *s)
+{
 	Parse_choice *p, *xp;
 	if (s == NULL) return;
 	for (p=s->first; p != NULL; p = xp) {
@@ -54,7 +54,8 @@ static void free_set(Parse_set *s) {
 
 static Parse_choice * make_choice(Parse_set *lset, int llw, int lrw, Connector * llc, Connector * lrc,
 						   Parse_set *rset, int rlw, int rrw, Connector * rlc, Connector * rrc,
-						   Disjunct *ld, Disjunct *md, Disjunct *rd) {
+						   Disjunct *ld, Disjunct *md, Disjunct *rd)
+{
 	Parse_choice *pc;
 	pc = (Parse_choice *) xalloc(sizeof(*pc));
 	pc->next = NULL;
@@ -74,9 +75,12 @@ static Parse_choice * make_choice(Parse_set *lset, int llw, int lrw, Connector *
 	return pc;
 }
 
-static void put_choice_in_set(Parse_set *s, Parse_choice *pc) {
-/* Put this parse_choice into a given set.  The current pointer is always
-   left pointing to the end of the list. */
+/**
+ * Put this parse_choice into a given set.  The current pointer is always
+ * left pointing to the end of the list.
+ */
+static void put_choice_in_set(Parse_set *s, Parse_choice *pc)
+{
 	if (s->first == NULL) {
 		s->first = pc;
 	} else {
@@ -86,7 +90,8 @@ static void put_choice_in_set(Parse_set *s, Parse_choice *pc) {
 	pc->next = NULL;
 }
 
-static int x_hash(int lw, int rw, Connector *le, Connector *re, int cost, Parse_info pi) {
+static int x_hash(int lw, int rw, Connector *le, Connector *re, int cost, Parse_info pi)
+{
 	int i;
 	i = 0;
 
@@ -98,10 +103,13 @@ static int x_hash(int lw, int rw, Connector *le, Connector *re, int cost, Parse_
 	return i & (pi->x_table_size-1);
 }
 
-void init_x_table(Sentence sent) {
-	/* A piecewise exponential function determines the size of the hash table.	  */
-	/* Probably should make use of the actual number of disjuncts, rather than just */
-	/* the number of words														  */
+/**
+ * A piecewise exponential function determines the size of the hash
+ * table.  Probably should make use of the actual number of disjuncts,
+ * rather than just the number of words.
+ */
+void init_x_table(Sentence sent)
+{
 	int i, x_table_size;
 	Parse_info pi;
 
@@ -126,10 +134,13 @@ void init_x_table(Sentence sent) {
 	}
 }
 
-static void free_x_table(Parse_info pi) {
-/* This is the function that should be used to free tha set structure. Since
-   it's a dag, a recursive free function won't work.  Every time we create
-   a set element, we put it in the hash table, so this is OK.*/
+/**
+ * This is the function that should be used to free tha set structure. Since
+ * it's a dag, a recursive free function won't work.  Every time we create
+ * a set element, we put it in the hash table, so this is OK.
+ */
+static void free_x_table(Parse_info pi)
+{
 	int i;
 	X_table_connector *t, *x;
 
@@ -151,9 +162,12 @@ static void free_x_table(Parse_info pi) {
 	pi->x_table = NULL;
 }
 
+/**
+ * Returns the pointer to this info, NULL if not there.
+ */
 static X_table_connector * x_table_pointer(int lw, int rw, Connector *le, Connector *re,
-									int cost, Parse_info pi) {
-	/* returns the pointer to this info, NULL if not there */
+									int cost, Parse_info pi)
+{
 	X_table_connector *t;
 	t = pi->x_table[x_hash(lw, rw, le, re, cost, pi)];
 	for (; t != NULL; t = t->next) {
@@ -162,7 +176,7 @@ static X_table_connector * x_table_pointer(int lw, int rw, Connector *le, Connec
 	return NULL;
 }
 
-#if 0
+#if DEAD_CODE
 Parse_set * x_table_lookup(int lw, int rw, Connector *le, Connector *re,
 						   int cost, Parse_info pi) {
 	/* returns the count for this quintuple if there, -1 otherwise */
@@ -172,9 +186,12 @@ Parse_set * x_table_lookup(int lw, int rw, Connector *le, Connector *re,
 }
 #endif
 
+/**
+ * Stores the value in the x_table.  Assumes it's not already there.
+ */
 static X_table_connector * x_table_store(int lw, int rw, Connector *le, Connector *re,
-								  int cost, Parse_set * set, Parse_info pi) {
-	/* Stores the value in the x_table.  Assumes it's not already there */
+								  int cost, Parse_set * set, Parse_info pi)
+{
 	X_table_connector *t, *n;
 	int h;
 
@@ -200,12 +217,14 @@ static void x_table_update(int lw, int rw, Connector *le, Connector *re,
 #endif
 
 
-/** 
+/**
  * returns NULL if there are no ways to parse, or returns a pointer
  * to a set structure representing all the ways to parse.
  */
-static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
-					  Connector *le, Connector *re, int cost, Parse_info pi)
+static Parse_set * parse_set(Sentence sent,
+                 Disjunct *ld, Disjunct *rd, int lw, int rw,
+					  Connector *le, Connector *re, int cost,
+                 int islands_ok, Parse_info pi)
 {
 	Disjunct * d, * dis;
 	int start_word, end_word, w;
@@ -220,7 +239,7 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 
 	assert(cost >= 0, "parse_set() called with cost < 0.");
 
-	count = table_lookup(lw, rw, le, re, cost);
+	count = table_lookup(sent, lw, rw, le, re, cost);
 
 	/*
 	  assert(count >= 0, "parse_set() called on params that were not in the table.");
@@ -253,9 +272,9 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 			return xt->set;
 		} else {
 			w = lw+1;
-			for (dis = local_sent[w].d; dis != NULL; dis = dis->next) {
+			for (dis = sent->word[w].d; dis != NULL; dis = dis->next) {
 				if (dis->left == NULL) {
-					rs[0] = parse_set(dis, NULL, w, rw, dis->right, NULL, cost-1, pi);
+					rs[0] = parse_set(sent, dis, NULL, w, rw, dis->right, NULL, cost-1, islands_ok, pi);
 					if (rs[0] == NULL) continue;
 					a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
 										   rs[0], w, rw, NULL, NULL,
@@ -263,7 +282,7 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 					put_choice_in_set(xt->set, a_choice);
 				}
 			}
-			rs[0] = parse_set(NULL, NULL, w, rw, NULL, NULL, cost-1, pi);
+			rs[0] = parse_set(sent, NULL, NULL, w, rw, NULL, NULL, cost-1, islands_ok, pi);
 			if (rs[0] != NULL) {
 				a_choice = make_choice(dummy_set(), lw, w, NULL, NULL,
 									   rs[0], w, rw, NULL, NULL,
@@ -288,7 +307,7 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 	}
 
 	for (w=start_word; w < end_word+1; w++) {
-		m1 = m = form_match_list(w, le, lw, re, rw);
+		m1 = m = form_match_list(sent, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next) {
 			d = m->d;
 			for (lcost = 0; lcost <= cost; lcost++) {
@@ -298,20 +317,20 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
 
-				Lmatch = (le != NULL) && (d->left != NULL) && match(le, d->left, lw, w);
-				Rmatch = (d->right != NULL) && (re != NULL) && match(d->right, re, w, rw);
+				Lmatch = (le != NULL) && (d->left != NULL) && match(sent, le, d->left, lw, w);
+				Rmatch = (d->right != NULL) && (re != NULL) && match(sent, d->right, re, w, rw);
 				for (i=0; i<4; i++) {ls[i] = rs[i] = NULL;}
 				if (Lmatch) {
-					ls[0] = parse_set(ld, d, lw, w, le->next, d->left->next, lcost, pi);
-					if (le->multi) ls[1] = parse_set(ld, d, lw, w, le, d->left->next, lcost, pi);
-					if (d->left->multi) ls[2] = parse_set(ld, d, lw, w, le->next, d->left, lcost, pi);
-					if (le->multi && d->left->multi) ls[3] = parse_set(ld, d, lw, w, le, d->left, lcost, pi);
+					ls[0] = parse_set(sent, ld, d, lw, w, le->next, d->left->next, lcost, islands_ok, pi);
+					if (le->multi) ls[1] = parse_set(sent, ld, d, lw, w, le, d->left->next, lcost, islands_ok, pi);
+					if (d->left->multi) ls[2] = parse_set(sent, ld, d, lw, w, le->next, d->left, lcost, islands_ok, pi);
+					if (le->multi && d->left->multi) ls[3] = parse_set(sent, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
 				}
 				if (Rmatch) {
-					rs[0] = parse_set(d, rd, w, rw, d->right->next, re->next, rcost, pi);
-					if (d->right->multi) rs[1] = parse_set(d, rd, w,rw,d->right,re->next, rcost, pi);
-					if (re->multi) rs[2] = parse_set(d, rd, w, rw, d->right->next, re, rcost, pi);
-					if (d->right->multi && re->multi) rs[3] = parse_set(d, rd, w, rw, d->right, re, rcost, pi);
+					rs[0] = parse_set(sent, d, rd, w, rw, d->right->next, re->next, rcost, islands_ok, pi);
+					if (d->right->multi) rs[1] = parse_set(sent, d, rd, w,rw,d->right,re->next, rcost, islands_ok, pi);
+					if (re->multi) rs[2] = parse_set(sent, d, rd, w, rw, d->right->next, re, rcost, islands_ok, pi);
+					if (d->right->multi && re->multi) rs[3] = parse_set(sent, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
 				}
 
 				for (i=0; i<4; i++) {
@@ -328,7 +347,7 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 
 				if (ls[0] != NULL || ls[1] != NULL || ls[2] != NULL || ls[3] != NULL) {
 					/* evaluate using the left match, but not the right */
-					rset = parse_set(d, rd, w, rw, d->right, re, rcost, pi);
+					rset = parse_set(sent, d, rd, w, rw, d->right, re, rcost, islands_ok, pi);
 					if (rset != NULL) {
 						for (i=0; i<4; i++) {
 							if (ls[i] == NULL) continue;
@@ -342,7 +361,7 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 				}
 				if ((le == NULL) && (rs[0] != NULL || rs[1] != NULL || rs[2] != NULL || rs[3] != NULL)) {
 					/* evaluate using the right match, but not the left */
-					lset = parse_set(ld, d, lw, w, le, d->left, lcost, pi);
+					lset = parse_set(sent, ld, d, lw, w, le, d->left, lcost, islands_ok, pi);
 
 					if (lset != NULL) {
 						for (i=0; i<4; i++) {
@@ -357,7 +376,7 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
 				}
 			}
 		}
-		put_match_list(m1);
+		put_match_list(sent, m1);
 	}
 	xt->set->current = xt->set->first;
 	return xt->set;
@@ -367,7 +386,7 @@ static Parse_set * parse_set(Disjunct *ld, Disjunct *rd, int lw, int rw,
  * return TRUE if and only if overflow in the number of parses occured.
  * Use a 64-bit int for counting.
  */
-static int verify_set_node(Parse_set *set) 
+static int verify_set_node(Parse_set *set)
 {
 	Parse_choice *pc;
 	s64 n;
@@ -415,11 +434,9 @@ int build_parse_set(Sentence sent, int cost, Parse_Options opts)
 {
 	Parse_set * whole_set;
 
-	local_sent = sent->word;
-	islands_ok = opts->islands_ok;
-
 	whole_set =
-		parse_set(NULL, NULL, -1, sent->length, NULL, NULL, cost+1, sent->parse_info);
+		parse_set(sent, NULL, NULL, -1, sent->length, NULL, NULL, cost+1,
+		          opts->islands_ok, sent->parse_info);
 
 	if ((whole_set != NULL) && (whole_set->current != NULL)) {
 		whole_set->current = whole_set->first;
@@ -427,11 +444,11 @@ int build_parse_set(Sentence sent, int cost, Parse_Options opts)
 
 	sent->parse_info->parse_set = whole_set;
 
-	local_sent = NULL;
 	return verify_set(sent->parse_info);
 }
 
-void free_parse_set(Sentence sent) {
+void free_parse_set(Sentence sent)
+{
 	/* This uses the x_table to free the whole parse set (the set itself
 	   cannot be used cause it's a dag).  called from the outside world */
 	if (sent->parse_info != NULL) {
@@ -450,7 +467,8 @@ static void initialize_links(Parse_info pi) {
 	}
 }
 
-static void issue_link(Parse_info pi, Disjunct * ld, Disjunct * rd, struct Link_s link) {
+static void issue_link(Parse_info pi, Disjunct * ld, Disjunct * rd, Link link)
+{
 	assert(pi->N_links <= MAX_LINKS-1, "Too many links");
 	pi->link_array[pi->N_links] = link;
 	pi->N_links++;
@@ -459,7 +477,8 @@ static void issue_link(Parse_info pi, Disjunct * ld, Disjunct * rd, struct Link_
 	pi->chosen_disjuncts[link.r] = rd;
 }
 
-static void issue_links_for_choice(Parse_info pi, Parse_choice *pc) {
+static void issue_links_for_choice(Parse_info pi, Parse_choice *pc)
+{
 	if (pc->link[0].lc != NULL) { /* there is a link to generate */
 		issue_link(pi, pc->ld, pc->md, pc->link[0]);
 	}
@@ -485,12 +504,15 @@ void build_current_linkage(Parse_info pi) {
 	build_current_linkage_recursive(pi, pi->parse_set);
 }
 
-static int advance_linkage(Parse_info pi, Parse_set * set) {
-	/* Advance the "current" linkage to the next one
-	   return 1 if there's a "carry" from this node,
-	   which indicates that the scan of this node has
-	   just been completed, and it's now back to it's
-	   starting state. */
+/**
+ * Advance the "current" linkage to the next one
+ * return 1 if there's a "carry" from this node,
+ * which indicates that the scan of this node has
+ * just been completed, and it's now back to it's
+ * starting state.
+ */
+static int advance_linkage(Parse_info pi, Parse_set * set)
+{
 	if (set == NULL) return 1;  /* probably can't happen */
 	if (set->first == NULL) return 1;  /* the empty set */
 	if (advance_linkage(pi, set->current->set[0]) == 1) {
@@ -506,7 +528,8 @@ static int advance_linkage(Parse_info pi, Parse_set * set) {
 }
 
 #ifdef UNUSED_FUNCTION
-static void advance_parse_set(Parse_info pi) {
+static void advance_parse_set(Parse_info pi)
+{
 	 advance_linkage(pi, pi->parse_set);
 }
 #endif
@@ -528,7 +551,8 @@ static void list_links(Parse_info pi, Parse_set * set, int index)
 	 list_links(pi, pc->set[1], index / pc->set[0]->count);
 }
 
-static void list_random_links(Parse_info pi, Parse_set * set) {
+static void list_random_links(Parse_info pi, Parse_set * set)
+{
 	 Parse_choice *pc;
 	 int num_pc, new_index;
 
@@ -538,7 +562,7 @@ static void list_random_links(Parse_info pi, Parse_set * set) {
 		  num_pc++;
 	 }
 
-	 new_index = my_random() % num_pc;
+	 new_index = rand_r(&pi->rand_state) % num_pc;
 
 	 num_pc = 0;
 	 for (pc = set->first; pc != NULL; pc = pc->next) {
@@ -552,15 +576,17 @@ static void list_random_links(Parse_info pi, Parse_set * set) {
 	 list_random_links(pi, pc->set[1]);
 }
 
-void extract_links(int index, int cost, Parse_info pi) {
-/* Generate the list of all links of the indexth parsing of the
-   sentence.  For this to work, you must have already called parse, and
-   already built the whole_set. */
+/**
+ * Generate the list of all links of the index'th parsing of the
+ * sentence.  For this to work, you must have already called parse, and
+ * already built the whole_set.
+ */
+void extract_links(int index, int cost, Parse_info pi)
+{
 	initialize_links(pi);
+	pi->rand_state = index;
 	if (index < 0) {
-		my_random_initialize(index);
 		list_random_links(pi, pi->parse_set);
-		my_random_finalize();
 	}
 	else {
 		list_links(pi, pi->parse_set, index);
