@@ -43,6 +43,10 @@
 #include <editline/readline.h>
 #endif 
 
+#ifdef _MSC_VER
+#define LINK_GRAMMAR_DLL_EXPORT 0
+#endif
+
 #include <link-grammar/link-includes.h>
 #include <link-grammar/structures.h>
 #include <link-grammar/error.h>
@@ -176,17 +180,17 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 		if (parse_options_get_display_on(opts)) {
 			string = linkage_print_diagram(linkage);
 			fprintf(stdout, "%s", string);
-			free(string);
+			linkage_free_diagram(string);
 		}
 		if (parse_options_get_display_links(opts)) {
 			string = linkage_print_links_and_domains(linkage);
 			fprintf(stdout, "%s", string);
-			free(string);
+			linkage_free_links_and_domains(string);
 		}
 		if (parse_options_get_display_postscript(opts)) {
 			string = linkage_print_postscript(linkage, FALSE);
 			fprintf(stdout, "%s\n", string);
-			free(string);
+			linkage_free_postscript(string);
 		}
 	}
 	if ((mode = parse_options_get_display_constituents(opts)))
@@ -194,7 +198,7 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 		string = linkage_print_constituent_tree(linkage, mode);
 		if (string != NULL) {
 			fprintf(stdout, "%s\n", string);
-			free(string);
+			linkage_free_constituent_tree_str(string);
 		} else {
 			fprintf(stderr, "Can't generate constituents.\n");
 			fprintf(stderr, "Constituent processing has been turned off.\n");
@@ -431,17 +435,17 @@ int main(int argc, char * argv[])
 
 	opts = parse_options_create();
 	if (opts == NULL) {
-		fprintf(stderr, "%s\n", lperrmsg);
+		fprintf(stderr, "%s: Fatal error: unable to create parse options\n", argv[0]);
 		exit(-1);
 	}
 
 	panic_parse_opts = parse_options_create();
 	if (panic_parse_opts == NULL) {
-		fprintf(stderr, "%s\n", lperrmsg);
+		fprintf(stderr, "%s: Fatal error: unable to create panic parse options\n", argv[0]);
 		exit(-1);
 	}
 	setup_panic_parse_options(panic_parse_opts);
-	parse_options_set_max_sentence_length(opts, 70);
+	parse_options_set_max_sentence_length(opts, 170);
 	parse_options_set_panic_mode(opts, TRUE);
 	parse_options_set_max_parse_time(opts, 30);
 	parse_options_set_linkage_limit(opts, 1000);
@@ -453,7 +457,7 @@ int main(int argc, char * argv[])
 		dict = dictionary_create_default_lang();
 
 	if (dict == NULL) {
-		fprintf(stderr, "%s\n", lperrmsg);
+		fprintf(stderr, "%s: Fatal error: Unable to open  dictionary.\n", argv[0]);
 		exit(-1);
 	}
 
@@ -492,11 +496,8 @@ int main(int argc, char * argv[])
 
 		sent = sentence_create(input_string, dict);
 
-		if (sent == NULL) {
-			if (verbosity > 0) fprintf(stderr, "%s\n", lperrmsg);
-			/* if (lperrno != NOTINDICT) exit(-1); */
-			continue;
-		}
+		if (sent == NULL) continue;
+
 		if (sentence_length(sent) > parse_options_get_max_sentence_length(opts)) {
 			if (verbosity > 0) {
 				fprintf(stdout,
