@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */ 
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 /***********************************************************************
  pp_knowledge.c
@@ -26,22 +26,25 @@
 
 /****************** non-exported functions ***************************/
 
-static void check_domain_is_legal(char *p) 
+static void check_domain_is_legal(char *p)
 {
-  if (strlen(p)>1)
-    error("post_process: Domain (%s) must be a single character", p);
+  if (strlen(p) > 1)
+  {
+    prt_error("Fatal Error: post_process(): Domain (%s) must be a single character", p);
+    exit(1);
+  }
 }
 
 static void initialize_set_of_links_starting_bounded_domain(pp_knowledge *k)
 {
   int i,j,d,domain_of_rule;
-  k->set_of_links_starting_bounded_domain = 
+  k->set_of_links_starting_bounded_domain =
     pp_linkset_open(PP_MAX_UNIQUE_LINK_NAMES);
-  for (i=0; k->bounded_rules[i].msg!=0; i++) 
+  for (i=0; k->bounded_rules[i].msg!=0; i++)
     {
       domain_of_rule = k->bounded_rules[i].domain;
-      for (j=0; (d=(k->starting_link_lookup_table[j].domain))!=-1; j++) 
-	if (d==domain_of_rule) 
+      for (j=0; (d=(k->starting_link_lookup_table[j].domain))!=-1; j++)
+	if (d==domain_of_rule)
 	  pp_linkset_add(k->set_of_links_starting_bounded_domain,
 		      k->starting_link_lookup_table[j].starting_link);
     }
@@ -49,28 +52,34 @@ static void initialize_set_of_links_starting_bounded_domain(pp_knowledge *k)
 
 static void read_starting_link_table(pp_knowledge *k)
 {
-  /* read table of [link, domain type]. 
-     This tells us what domain type each link belongs to. 
+  /* read table of [link, domain type].
+     This tells us what domain type each link belongs to.
      This lookup table *must* be defined in the knowledge file. */
   char *p;
   const char label[] = "STARTING_LINK_TYPE_TABLE";
   int i, n_tokens;
   if (!pp_lexer_set_label(k->lt, label))
-    error("post_process: Couldn't find starting link table %s",label);
-  n_tokens = pp_lexer_count_tokens_of_label(k->lt); 
-  if (n_tokens %2) 
-    error("post_process: Link table must have format [<link> <domain name>]+");
+  {
+    prt_error("Fatal error: post_process: Couldn't find starting link table %s",label);
+    exit(1);
+  }
+  n_tokens = pp_lexer_count_tokens_of_label(k->lt);
+  if (n_tokens %2)
+  {
+    prt_error("Fatal error: post_process: Link table must have format [<link> <domain name>]+");
+    exit(1);
+  }
   k->nStartingLinks = n_tokens/2;
-  k->starting_link_lookup_table = (StartingLinkAndDomain*) 
+  k->starting_link_lookup_table = (StartingLinkAndDomain*)
     xalloc((1+k->nStartingLinks)*sizeof(StartingLinkAndDomain));
   for (i=0; i<k->nStartingLinks; i++)
     {
       /* read the starting link itself */
-      k->starting_link_lookup_table[i].starting_link = 
+      k->starting_link_lookup_table[i].starting_link =
 	string_set_add(pp_lexer_get_next_token_of_label(k->lt),k->string_set);
-      
+
       /* read the domain type of the link */
-      p = pp_lexer_get_next_token_of_label(k->lt);   
+      p = pp_lexer_get_next_token_of_label(k->lt);
       check_domain_is_legal(p);
       k->starting_link_lookup_table[i].domain = (int) p[0];
     }
@@ -92,9 +101,9 @@ static pp_linkset *read_link_set(pp_knowledge *k,
       printf("PP warning: Link set %s not defined: assuming empty.\n",label);
     n_strings = 0;
   }
-  else n_strings = pp_lexer_count_tokens_of_label(k->lt); 
-  ls = pp_linkset_open(n_strings); 
-  for (i=0; i<n_strings; i++) 
+  else n_strings = pp_lexer_count_tokens_of_label(k->lt);
+  ls = pp_linkset_open(n_strings);
+  for (i=0; i<n_strings; i++)
     pp_linkset_add(ls,
 		   string_set_add(pp_lexer_get_next_token_of_label(k->lt),ss));
   return ls;
@@ -108,7 +117,7 @@ static void read_link_sets(pp_knowledge *k)
   k->domain_contains_links    =read_link_set(k,"DOMAIN_CONTAINS_LINKS",ss);
   k->ignore_these_links       =read_link_set(k,"IGNORE_THESE_LINKS",ss);
   k->restricted_links         =read_link_set(k,"RESTRICTED_LINKS",ss);
-  k->must_form_a_cycle_links  =read_link_set(k,"MUST_FORM_A_CYCLE_LINKS",ss); 
+  k->must_form_a_cycle_links  =read_link_set(k,"MUST_FORM_A_CYCLE_LINKS",ss);
   k->urfl_only_domain_starter_links=
       read_link_set(k,"URFL_ONLY_DOMAIN_STARTER_LINKS",ss);
   k->left_domain_starter_links=read_link_set(k,"LEFT_DOMAIN_STARTER_LINKS",ss);
@@ -130,7 +139,7 @@ static void read_connected_rule(pp_knowledge *k, const char *label)
 {
   /* This is a degenerate class of rules: either a single rule asserting
      connectivity is there, or it isn't. The only information in the
-     rule (besides its presence) is the error message to display if 
+     rule (besides its presence) is the error message to display if
      the rule is violated */
   k->connected_rules = (pp_rule *) xalloc (sizeof(pp_rule));
   if (!pp_lexer_set_label(k->lt, label))
@@ -139,8 +148,11 @@ static void read_connected_rule(pp_knowledge *k, const char *label)
       if (verbosity>0) printf("PP warning: Not using 'link is connected' rule\n");
       return;
     }
-  if (pp_lexer_count_tokens_of_label(k->lt)>1) 
-    error("post_process: Invalid syntax in %s", label);
+  if (pp_lexer_count_tokens_of_label(k->lt)>1)
+  {
+    prt_error("Fatal Error: post_process(): Invalid syntax in %s", label);
+    exit(1);
+  }
   k->connected_rules[0].msg =
     string_set_add(pp_lexer_get_next_token_of_label(k->lt), k->string_set);
 }
@@ -152,12 +164,12 @@ static void read_form_a_cycle_rules(pp_knowledge *k, const char *label)
   pp_linkset *lsHandle;
   char **tokens;
   if (!pp_lexer_set_label(k->lt, label)) {
-      k->n_form_a_cycle_rules = 0;             
+      k->n_form_a_cycle_rules = 0;
       if (verbosity>0)
 	printf("PP warning: Not using any 'form a cycle' rules\n");
   }
   else {
-    n_commas = pp_lexer_count_commas_of_label(k->lt); 
+    n_commas = pp_lexer_count_commas_of_label(k->lt);
     k->n_form_a_cycle_rules = (n_commas + 1)/2;
   }
   k->form_a_cycle_rules=
@@ -165,17 +177,24 @@ static void read_form_a_cycle_rules(pp_knowledge *k, const char *label)
   for (r=0; r<k->n_form_a_cycle_rules; r++)
     {
       /* read link set */
-      tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens); 
-      if (n_tokens <= 0) error("syntax error in knowledge file");
+      tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
+      if (n_tokens <= 0)
+      {
+        prt_error("Fatal Error: syntax error in knowledge file");
+        exit(1);
+      }
       lsHandle = pp_linkset_open(n_tokens);
-      for (i=0; i<n_tokens; i++) 
-	pp_linkset_add(lsHandle,string_set_add(tokens[i], k->string_set));
+      for (i=0; i<n_tokens; i++)
+          pp_linkset_add(lsHandle,string_set_add(tokens[i], k->string_set));
       k->form_a_cycle_rules[r].link_set=lsHandle;
-      
+
       /* read error message */
       tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
-      if (n_tokens>1)
-	error("post_process: Invalid syntax (rule %i of %s)",r+1,label);
+      if (n_tokens > 1)
+      {
+         prt_error("Fatal Error: post_process: Invalid syntax (rule %i of %s)",r+1,label);
+         exit(1);
+      }
       k->form_a_cycle_rules[r].msg=string_set_add(tokens[0],k->string_set);
     }
 
@@ -192,33 +211,39 @@ static void read_bounded_rules(pp_knowledge *k, const char *label)
       if (verbosity>0) printf("PP warning: Not using any 'bounded' rules\n");
   }
   else {
-    n_commas = pp_lexer_count_commas_of_label(k->lt); 
+    n_commas = pp_lexer_count_commas_of_label(k->lt);
     k->n_bounded_rules = (n_commas + 1)/2;
   }
   k->bounded_rules = (pp_rule*) xalloc ((1+k->n_bounded_rules)*sizeof(pp_rule));
   for (r=0; r<k->n_bounded_rules; r++)
     {
       /* read domain */	
-      tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens); 
+      tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
       if (n_tokens!=1)
-	error("post_process: Invalid syntax: rule %i of %s",r+1,label);
+      {
+        prt_error("Fatal Error: post_process: Invalid syntax: rule %i of %s",r+1,label);
+        exit(1);
+      }
       k->bounded_rules[r].domain = (int) tokens[0][0];
-      
+
       /* read error message */
-      tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens); 
-      if (n_tokens!=1) 
-	error("post_process: Invalid syntax: rule %i of %s",r+1,label);
+      tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
+      if (n_tokens!=1)
+      {
+        prt_error("Fatal Error: post_process: Invalid syntax: rule %i of %s",r+1,label);
+        exit(1);
+      }
       k->bounded_rules[r].msg = string_set_add(tokens[0], k->string_set);
     }
-  
+
   /* sentinel entry */
   k->bounded_rules[k->n_bounded_rules].msg = 0;
 }
 
-static void read_contains_rules(pp_knowledge *k, const char *label, 
+static void read_contains_rules(pp_knowledge *k, const char *label,
 				pp_rule **rules, int *nRules)
 {
-  /* Reading the 'contains_one_rules' and reading the 
+  /* Reading the 'contains_one_rules' and reading the
      'contains_none_rules' into their respective arrays */
   int n_commas, n_tokens, i, r;
   char *p, **tokens;
@@ -235,30 +260,36 @@ static void read_contains_rules(pp_knowledge *k, const char *label,
     {
       /* first read link */
       tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
-      if (n_tokens>1) 
-	error("post_process: Invalid syntax in %s (rule %i)",label,r+1);
+      if (n_tokens>1)
+      {
+        prt_error("Fatal Error: post_process: Invalid syntax in %s (rule %i)",label,r+1);
+        exit(1);
+      }
       (*rules)[r].selector = string_set_add(tokens[0], k->string_set);
-      
+
       /* read link set */
       tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
       (*rules)[r].link_set = pp_linkset_open(n_tokens);
       (*rules)[r].link_set_size = n_tokens;
       (*rules)[r].link_array = (char **) xalloc((1+n_tokens)*sizeof(char*));
-      for (i=0; i<n_tokens; i++) 
+      for (i=0; i<n_tokens; i++)
 	{
 	  p=string_set_add(tokens[i], k->string_set);
-	  pp_linkset_add((*rules)[r].link_set, p); 
+	  pp_linkset_add((*rules)[r].link_set, p);
 	  (*rules)[r].link_array[i]=p;
 	}
       (*rules)[r].link_array[i]=0; /* NULL-terminator */
 
       /* read error message */
       tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
-      if (n_tokens>1) 
-	error("post_process: Invalid syntax in %s (rule %i)",label,r+1);
+      if (n_tokens>1)
+      {
+        prt_error("Fatal Error: post_process: Invalid syntax in %s (rule %i)",label,r+1);
+        exit(1);
+      }
       (*rules)[r].msg = string_set_add(tokens[0], k->string_set);
     }
-  
+
   /* sentinel entry */
   (*rules)[*nRules].msg = 0;
 }
@@ -270,9 +301,9 @@ static void read_rules(pp_knowledge *k)
   read_connected_rule(k, "CONNECTED_RULES");
   read_bounded_rules(k,  "BOUNDED_RULES");
   read_contains_rules(k, "CONTAINS_ONE_RULES" ,
-		      &(k->contains_one_rules), &(k->n_contains_one_rules)); 
+		      &(k->contains_one_rules), &(k->n_contains_one_rules));
   read_contains_rules(k, "CONTAINS_NONE_RULES",
-		      &(k->contains_none_rules), &(k->n_contains_none_rules)); 
+		      &(k->contains_none_rules), &(k->n_contains_none_rules));
 }
 
 static void free_rules(pp_knowledge *k)
@@ -291,7 +322,7 @@ static void free_rules(pp_knowledge *k)
     pp_linkset_close(rule->link_set);
   }
 
-  for (r=0; r<k->n_form_a_cycle_rules; r++) 
+  for (r=0; r<k->n_form_a_cycle_rules; r++)
     pp_linkset_close(k->form_a_cycle_rules[r].link_set);
   xfree((void*)k->bounded_rules,           rs*(1+k->n_bounded_rules));
   xfree((void*)k->connected_rules,         rs);
@@ -307,7 +338,11 @@ pp_knowledge *pp_knowledge_open(char *path)
   /* read knowledge from disk into pp_knowledge */
   FILE *f=dictopen(path, "r");
   pp_knowledge *k = (pp_knowledge *) xalloc (sizeof(pp_knowledge));
-  if (!f) error("Couldn't find post-process knowledge file %s", path);
+  if (!f)
+  {
+    prt_error("Fatal Error: Couldn't find post-process knowledge file %s", path);
+    exit(1);
+  }
   k->lt = pp_lexer_open(f);
   fclose(f);
   k->string_set = string_set_create();
