@@ -4,7 +4,7 @@
 /* All rights reserved                                                          */
 /*                                                                              */
 /* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */ 
+/* license set forth in the LICENSE file included with this software,           */
 /* and also available at http://www.link.cs.cmu.edu/link/license.html           */
 /* This license allows free redistribution and use in source and binary         */
 /* forms, with or without modification, subject to certain conditions.          */
@@ -16,3 +16,5 @@ void free_deletable(Sentence sent);
 void free_effective_dist(Sentence sent);
 void prepare_to_parse(Sentence sent, Parse_Options opts);
 void install_fat_connectors(Sentence sent);
+void build_deletable(Sentence sent, int has_conjunction);
+void build_effective_dist(Sentence sent, int has_conjunction);
