@@ -272,7 +272,7 @@ static void insert_S(connector_table *ct, Connector * c)
 			return;
 		}
 	}
-	e = init_connector((Connector *) xalloc(sizeof(Connector)));
+	e = connector_new();
 	*e = *c;
 	e->next = ct->table[h];
 	ct->table[h] = e;
@@ -940,7 +940,6 @@ static int mark_dead_connectors(connector_table *ct, Exp * e, int dir)
 	init_connector(&dummy);
 	dummy.label = NORMAL_LABEL;
 	dummy.priority = THIN_priority;
-	/*	dummy.my_word = NO_WORD; */  /* turn off the length part of the matching */
 	count = 0;
 	if (e->type == CONNECTOR_type) {
 		if (e->dir == dir) {
@@ -968,7 +967,6 @@ static void insert_connectors(connector_table *ct, Exp * e, int dir)
 	init_connector(&dummy);
 	dummy.label = NORMAL_LABEL;
 	dummy.priority = THIN_priority;
-	/*	dummy.my_word = NO_WORD; */ /* turn off the length part of the matching */
 
 	if (e->type == CONNECTOR_type) {
 		if (e->dir == dir) {
