@@ -25,13 +25,16 @@
   The dictionary file is a sequence of ENTRIES.  Each ENTRY is one or
   more WORDS (a sequence of upper or lower case letters) separated by
   spaces, followed by a ":", followed by an EXPRESSION followed by a
-  ";".  An EXPRESSION is a lisp expression where the functions are "&"
+  ";".  An EXPRESSION is an expression where the operators are "&"
   or "and" or "|" or "or", and there are three types of parentheses:
   "()", "{}", and "[]".  The terminal symbols of this grammar are the
-  connectors, which are strings of letters or numbers or *s.  (This
-  description applies to the prefix form of the dictionary.  the current
-  dictionary is written in infix form.  If the defined constant
-  INFIX_NOTATION is defined, then infix is used otherwise prefix is used.)
+  connectors, which are strings of letters or numbers or *s.
+  Expressions may be written in prefix or infix form. In prefix-form,
+  the expressions are lisp-like, with the operators &, | preceeding
+  the operands. In infix-form, the operators are in the middle. The
+  current dictionaries are in infix form.  If the C preprocessor
+  constant INFIX_NOTATION is defined, then the dictionary is assumed
+  to be in infix form.
 
   The connector begins with an optinal @, which is followed by an upper
   case sequence of letters. Each subsequent *, lower case letter or
@@ -85,10 +88,15 @@
   "ID".  This is reserved for the connectors automatically
   generated for idioms.
 
+  Dictionary words may be followed by a dot (period, "."), and a "subscript"
+  identifying the word type. The subscript may be one or more letters or 
+  numbers, but must begin with a letter. Currently, the dictionary contains
+  (mostly?) subscripts consisting of a single letter, and these serve mostly
+  to identify the part-of-speech. In general, subscripts can also be used
+  to distinguish different word senses.
 */
 
 static int link_advance(Dictionary dict);
-static Dict_node * abridged_lookup_list(Dictionary dict, const char *s);
 
 static void dict_error2(Dictionary dict, const char * s, const char *s2)
 {
@@ -134,17 +142,6 @@ static void warning(Dictionary dict, const char * s)
 	          s, dict->line_number, dict->token);
 }
 
-Exp * Exp_create(Dictionary dict)
-{
-	/* allocate a new Exp node and link it into the
-	   exp_list for freeing later */
-	Exp * e;
-	e = (Exp *) xalloc(sizeof(Exp));
-	e->next = dict->exp_list;
-	dict->exp_list = e;
-	return e;
-}
-
 /**
  * This gets the next character from the input, eliminating comments.
  * If we're in quote mode, it does not consider the % character for
@@ -165,11 +162,12 @@ static wint_t get_character(Dictionary dict, int quote_mode)
 
 /*
  * This set of 10 characters are the ones defining the syntax of the dictionary.
-*/
+ */
 #define SPECIAL "(){};[]&|:"
 
 /**
  * This reads the next token from the input into token.
+ * Return 1 if a character was read, else return 0 (and print a warning).
  */
 static int link_advance(Dictionary dict)
 {
@@ -179,7 +177,8 @@ static int link_advance(Dictionary dict)
 
 	dict->is_special = FALSE;
 
-	if (dict->already_got_it != '\0') {
+	if (dict->already_got_it != '\0')
+	{
 		dict->is_special = (strchr(SPECIAL, dict->already_got_it) != NULL);
 		if (dict->already_got_it == EOF) {
 			dict->token[0] = '\0';
@@ -191,12 +190,13 @@ static int link_advance(Dictionary dict)
 		return 1;
 	}
 
-	do { c = get_character(dict, FALSE); } while (iswspace(c)); 
+	do { c = get_character(dict, FALSE); } while (iswspace(c));
 
 	quote_mode = FALSE;
 
 	i = 0;
-	for (;;) {
+	for (;;)
+	{
 		if (i > MAX_TOKEN_LENGTH-3) {  /* 3 for multi-byte tokens */
 			dict_error(dict, "Token too long");
 			return 0;
@@ -212,7 +212,7 @@ static int link_advance(Dictionary dict)
 				return 0;
 			}
 
-			/* store UTF8 internally, always. */
+			/* Although we read wide chars, we store UTF8 internally, always. */
 			nr = wctomb(&dict->token[i], c);
 			if (nr < 0) {
 #ifndef _WIN32
@@ -276,16 +276,20 @@ static int link_advance(Dictionary dict)
 /**
  * Returns TRUE if this token is a special token and it is equal to c
  */
-static int is_equal(Dictionary dict, wint_t c)
+static inline int is_equal(Dictionary dict, wint_t c)
 {
-	return (dict->is_special && 
-	        wctob(c) == dict->token[0] && 
+	return (dict->is_special &&
+	        wctob(c) == dict->token[0] &&
 	        dict->token[1] == '\0');
 }
 
+/** 
+ * Make sure the string s is a valid connector.
+ * Return 1 if the connector is valid, else return 0,
+ * and print an appropriate warning message.
+ */
 static int check_connector(Dictionary dict, const char * s)
 {
-	/* makes sure the string s is a valid connector */
 	int i;
 	i = strlen(s);
 	if (i < 1) {
@@ -302,7 +306,7 @@ static int check_connector(Dictionary dict, const char * s)
 		dict_error(dict, "The first letter of a connector must be in [A--Z].");
 		return 0;
 	}
-	if ((*s=='I') && (*(s+1)=='D')) {
+	if ((*s == 'I') && (*(s+1) == 'D')) {
 		dict_error(dict, "Connectors beginning with \"ID\" are forbidden");
 		return 0;
 	}
@@ -316,7 +320,283 @@ static int check_connector(Dictionary dict, const char * s)
 	return 1;
 }
 
-Exp * make_unary_node(Dictionary dict, Exp * e);
+/* ======================================================================== */
+/**
+ * Dictionary entry comparison and ordering functions.
+ *
+ * The data structure storing the dictionary is simply a binary tree.
+ * The entries in the binary tree are sorted by alphabetical order.
+ * There is one catch, however: words may have suffixes (a dot, followed
+ * by the suffix), and these suffixes are to be handled appripriately
+ * during sorting and comparison.
+ *
+ * The use of suffixes means that the ordering of the words is not
+ * exactly the order given by strcmp.  The order must be such that, for
+ * example, "make" < "make.n" < "make-up" -- suffixed words come after
+ * the bare words, but before any other other words with non-ascii-alpha
+ * characters (such as the hyphen in "make-up", or possibly UTF8
+ * characters). Thus, stright "strcmp" can't be used to determine 
+ * dictionary order. 
+ *
+ * Thus, a set of specialized string comparison and ordering functions
+ * are provided. These "do the right thing" when matching string with
+ * and without suffixes.
+ */
+/** 
+ * dict_order - order two dictionary words in proper sort order.
+ * Return zero if the strings match, else return standard 
+ * (locale-dependent) UTF8 sort order.
+ */
+/* verbose version */
+/*
+int dict_order(char *s, char *t)
+{
+	int ss, tt;
+	while (*s != '\0' && *s == *t) {
+		s++;
+		t++;
+	}
+	if (*s == '.') {
+		ss = 1;
+	} else {
+		ss = (*s)<<1;
+	}
+	if (*t == '.') {
+		tt = 1;
+	} else {
+		tt = (*t)<<1;
+	}
+	return (ss - tt);
+}
+*/
+
+/* terse version */
+static int dict_order(const char *s, const char *t)
+{
+	while (*s != '\0' && *s == *t) {s++; t++;}
+	return (((*s == '.')?(1):((*s)<<1))  -  ((*t == '.')?(1):((*t)<<1)));
+}
+
+/**
+ * dict_order_wild() -- order dictionary strings, with wildcard.
+ * Assuming that s is a pointer to a dictionary string, and that
+ * t is a pointer to a search string, this returns 0 if they
+ * match, >0 if s>t, and <0 if s<t.
+ *
+ * The matching is done as follows.  Walk down the strings until
+ * you come to the end of one of them, or until you find unequal
+ * characters.  A "*" matches anything.  Otherwise, replace "."
+ * by "\0", and take the difference.  This behavior matches that
+ * of the function dict_order().
+ */
+static int dict_order_wild(const char * s, const char * t)
+{
+	while((*s != '\0') && (*s == *t)) {s++; t++;}
+	if ((*s == '*') || (*t == '*')) return 0;
+	return (((*s == '.')?('\0'):(*s))  -  ((*t == '.')?('\0'):(*t)));
+}
+
+/**
+ * dict_match --  return true if strings match, else false.
+ * A "bare" string (one without a suffix) will match any corresponding 
+ * string with a suffix; so, for example, "make" and "make.n" are 
+ * a match.  If both strings have suffixes, then the suffixes must match.
+ *
+ * A subscript is the part that followes the last "." in the word, and 
+ * that does not begin with a digit.
+ */
+static int dict_match(const char * s, const char * t)
+{
+	char *ds, *dt;
+	ds = strrchr(s, '.');
+	dt = strrchr(t, '.');
+
+	/* a dot at the end or a dot followed by a number is NOT
+	 * considered a subscript */
+	if ((dt != NULL) && ((*(dt+1) == '\0') ||
+	    (isdigit((int)*(dt+1))))) dt = NULL;
+	if ((ds != NULL) && ((*(ds+1) == '\0') ||
+	    (isdigit((int)*(ds+1))))) ds = NULL;
+
+	/* dt is NULL when there's no prefix ... */
+	if (dt == NULL && ds != NULL) {
+		if (((int)strlen(t)) > (ds-s)) return FALSE;   /* we need to do this to ensure that */
+		return (strncmp(s, t, ds-s) == 0);	           /* "i.e." does not match "i.e" */
+	} else if (dt != NULL && ds == NULL) {
+		if (((int)strlen(s)) > (dt-t)) return FALSE;
+		return (strncmp(s, t, dt-t) == 0);
+	} else {
+		return (strcmp(s, t) == 0);
+	}
+}
+
+/* ======================================================================== */
+
+static inline Dict_node * dict_node_new(void)
+{
+	return (Dict_node*) xalloc(sizeof(Dict_node));
+}
+
+static inline void free_dict_node(Dict_node *dn)
+{
+	xfree((char *)dn, sizeof(Dict_node));
+}
+
+/**
+ * prune_lookup_list -- discard all list entries that don't match string
+ * Walk the lookup list (of right links), discarding all nodes that do
+ * not match the dictionary string s. The matching is dictionary matching:
+ * suffixed entries will match "bare" entries.
+ */
+static Dict_node * prune_lookup_list(Dict_node *llist, const char * s)
+{
+	Dict_node *dn, *dnx, *list_new;
+
+	list_new = NULL;
+	for (dn = llist; dn != NULL; dn = dnx)
+	{
+		dnx = dn->right;
+		/* now put dn onto the answer list, or free it */
+		if (dict_match(dn->string, s))
+		{
+			dn->right = list_new;
+			list_new = dn;
+		}
+		else
+		{
+			free_dict_node(dn);
+		}
+	}
+
+	/* now reverse the list back */
+	llist = NULL;
+	for (dn = list_new; dn != NULL; dn = dnx)
+	{
+		dnx = dn->right;
+		dn->right = llist;
+		llist = dn;
+	}
+	return llist;
+}
+
+void free_lookup_list(Dict_node *llist)
+{
+	Dict_node * n;
+	while(llist != NULL)
+	{
+		n = llist->right;
+		free_dict_node(llist);
+		llist = n;
+	}
+}
+
+static void free_dict_node_recursive(Dict_node * dn)
+{
+	if (dn == NULL) return;
+	free_dict_node_recursive(dn->left);
+	free_dict_node_recursive(dn->right);
+	free_dict_node(dn);
+}
+
+/* ======================================================================== */
+/**
+ * rdictionary_lookup() -- recursive dictionary lookup
+ * Walk binary tree, given by 'dn', looking for the string 's'.
+ * For every node in the tree where 's' matches (including wildcards)
+ * make a copy of that node, and append it to llist.
+ */
+static Dict_node * rdictionary_lookup(Dict_node *llist,
+                                      Dict_node * dn, const char * s, int match_idiom)
+{
+	/* see comment in dictionary_lookup below */
+	int m;
+	Dict_node * dn_new;
+	if (dn == NULL) return llist;
+	m = dict_order_wild(s, dn->string);
+	if (m >= 0)
+	{
+		llist = rdictionary_lookup(llist, dn->right, s, match_idiom);
+	}
+	if ((m == 0) && (match_idiom || !is_idiom_word(dn->string)))
+	{
+		dn_new = dict_node_new();
+		*dn_new = *dn;
+		dn_new->right = llist;
+		llist = dn_new;
+	}
+	if (m <= 0)
+	{
+		llist = rdictionary_lookup(llist, dn->left, s, match_idiom);
+	}
+	return llist;
+}
+
+/**
+ * dictionary_lookup_list() - return lookup list of words in the dictionary
+ *
+ * Returns a pointer to a lookup list of the words in the dictionary.
+ * This list is made up of Dict_nodes, linked by their right pointers.
+ * The node, file and string fields are copied from the dictionary.
+ *
+ * The returned list must be freed with free_lookup_list().
+ */
+Dict_node * dictionary_lookup_list(Dictionary dict, const char *s)
+{
+   Dict_node * llist = rdictionary_lookup(NULL, dict->root, s, TRUE);
+   llist = prune_lookup_list(llist, s);
+   return llist;
+}
+
+static Dict_node * abridged_lookup_list(Dictionary dict, const char *s)
+{
+	Dict_node *llist;
+   llist = rdictionary_lookup(NULL, dict->root, s, FALSE);
+   llist = prune_lookup_list(llist, s);
+   return llist;
+}
+
+int boolean_dictionary_lookup(Dictionary dict, const char *s)
+{
+	Dict_node *llist = dictionary_lookup_list(dict, s);
+	int bool = (llist != NULL);
+	free_lookup_list(llist);
+	return bool;
+}
+
+/* ======================================================================== */
+/**
+ * Allocate a new Exp node and link it into the exp_list for freeing later.
+ */
+Exp * Exp_create(Dictionary dict)
+{
+	Exp * e;
+	e = (Exp *) xalloc(sizeof(Exp));
+	e->next = dict->exp_list;
+	dict->exp_list = e;
+	return e;
+}
+
+static inline void exp_free(Exp * e)
+{
+	xfree((char *)e, sizeof(Exp));
+}
+
+/* ======================================================================== */
+/**
+ * This creates a node with one child (namely e).  Initializes
+ * the cost to zero.
+ */
+static Exp * make_unary_node(Dictionary dict, Exp * e)
+{
+	Exp * n;
+	n = Exp_create(dict);
+	n->type = AND_type;  /* these must be AND types */
+	n->cost = 0;
+	n->u.l = (E_list *) xalloc(sizeof(E_list));
+	n->u.l->next = NULL;
+	n->u.l->e = e;
+	return n;
+}
 
 /**
  * connector() -- make a node for a connector or dictionary word.
@@ -329,11 +609,14 @@ static Exp * connector(Dictionary dict)
 	Dict_node *dn, *dn_head;
 	int i;
 
-	i = strlen(dict->token)-1;  /* this must be + or - if a connector */
-	if ((dict->token[i] != '+') && (dict->token[i] != '-')) {
+	i = strlen(dict->token) - 1;  /* this must be + or - if a connector */
+	if ((dict->token[i] != '+') && (dict->token[i] != '-'))
+	{
+		/* If we are here, token is a word */
 		dn_head = abridged_lookup_list(dict, dict->token);
 		dn = dn_head;
-		while((dn != NULL) && (strcmp(dn->string, dict->token) != 0)) {
+		while ((dn != NULL) && (strcmp(dn->string, dict->token) != 0))
+		{
 			dn = dn->right;
 		}
 		if (dn == NULL)
@@ -346,46 +629,40 @@ static Exp * connector(Dictionary dict)
 		}
 		n = make_unary_node(dict, dn->exp);
 		free_lookup_list(dn_head);
-	} else {
-		if (!check_connector(dict, dict->token)) {
+	} 
+	else
+	{
+		/* If we are here, token is a connector */
+		if (!check_connector(dict, dict->token))
+		{
 			return NULL;
 		}
 		n = Exp_create(dict);
 		n->dir = dict->token[i];
 		dict->token[i] = '\0';				   /* get rid of the + or - */
-		if (dict->token[0] == '@') {
+		if (dict->token[0] == '@')
+		{
 			n->u.string = string_set_add(dict->token+1, dict->string_set);
 			n->multi = TRUE;
-		} else {
+		}
+		else
+		{
 			n->u.string = string_set_add(dict->token, dict->string_set);
 			n->multi = FALSE;
 		}
 		n->type = CONNECTOR_type;
 		n->cost = 0;
 	}
-	if (!link_advance(dict)) {
+
+	if (!link_advance(dict))
+	{
+		exp_free(n);
 		return NULL;
 	}
 	return n;
 }
 
-/** 
- * This creates a node with one child (namely e).  Initializes
- * the cost to zero.
- */
-Exp * make_unary_node(Dictionary dict, Exp * e)
-{
-	Exp * n;
-	n = Exp_create(dict);
-	n->type = AND_type;  /* these must be AND types */
-	n->cost = 0;
-	n->u.l = (E_list *) xalloc(sizeof(E_list));
-	n->u.l->next = NULL;
-	n->u.l->e = e;
-	return n;
-}
-
-/** 
+/**
  * This creates a node with zero children.  Initializes
  * the cost to zero.
  */
@@ -419,11 +696,12 @@ static Exp * make_optional_node(Dictionary dict, Exp * e)
 	return n;
 }
 
-Exp * expression(Dictionary dict);
+/* ======================================================================== */
 
 #if ! defined INFIX_NOTATION
 
-/** 
+Exp * expression(Dictionary dict);
+/**
  * We're looking at the first of the stuff after an "and" or "or".
  * Build a Exp node for this expression.  Set the cost and optional
  * fields to the default values.  Set the type field according to type
@@ -454,7 +732,7 @@ Exp * operator_exp(Dictionary dict, int type)
 	return n;
 }
 
-/** 
+/**
  * Looks for the stuff that is allowed to be inside of parentheses
  * either & or | followed by a list, or a terminal symbol.
  */
@@ -477,7 +755,7 @@ Exp * in_parens(Dictionary dict)
 	}
 }
 
-/** 
+/**
  * Build (and return the root of) the tree for the expression beginning
  * with the current token.  At the end, the token is the first one not
  * part of this expression.
@@ -538,27 +816,28 @@ Exp * expression(Dictionary dict)
 	return n;
 }
 
-#else
+/* ======================================================================== */
+#else /* This is for infix notation */
 
-/* This is for infix notation */
-
-Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok);
+static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok);
 
 /**
  * Build (and return the root of) the tree for the expression beginning
  * with the current token.  At the end, the token is the first one not
  * part of this expression.
  */
-Exp * expression(Dictionary dict)
+static Exp * expression(Dictionary dict)
 {
-	return restricted_expression(dict, TRUE,TRUE);
+	return restricted_expression(dict, TRUE, TRUE);
 }
 
-Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
+static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 {
-	Exp * nl=NULL, * nr, * n;
+	Exp *nl = NULL, *nr;
 	E_list *ell, *elr;
-	if (is_equal(dict, '(')) {
+
+	if (is_equal(dict, '('))
+	{
 		if (!link_advance(dict)) {
 			return NULL;
 		}
@@ -573,7 +852,9 @@ Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 		if (!link_advance(dict)) {
 			return NULL;
 		}
-	} else if (is_equal(dict, '{')) {
+	}
+	else if (is_equal(dict, '{'))
+	{
 		if (!link_advance(dict)) {
 			return NULL;
 		}
@@ -589,7 +870,9 @@ Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 			return NULL;
 		}
 		nl = make_optional_node(dict, nl);
-	} else if (is_equal(dict, '[')) {
+	}
+	else if (is_equal(dict, '['))
+	{
 		if (!link_advance(dict)) {
 			return NULL;
 		}
@@ -605,27 +888,36 @@ Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 			return NULL;
 		}
 		nl->cost += 1;
-	} else if (!dict->is_special) {
+	}
+	else if (!dict->is_special)
+	{
 		nl = connector(dict);
 		if (nl == NULL) {
 			return NULL;
 		}
-	} else if (is_equal(dict, ')') || is_equal(dict, ']')) {
+	}
+	else if (is_equal(dict, ')') || is_equal(dict, ']'))
+	{
 		/* allows "()" or "[]" */
 		nl = make_zeroary_node(dict);
-	} else {
-			dict_error(dict, "Connector, \"(\", \"[\", or \"{\" expected.");
+	}
+	else
+	{
+		dict_error(dict, "Connector, \"(\", \"[\", or \"{\" expected.");
 		return NULL;
 	}
 
-	if (is_equal(dict, '&') || (strcmp(dict->token, "and")==0)) {
+	if (is_equal(dict, '&') || (strcmp(dict->token, "and") == 0))
+	{
+		Exp *n;
+
 		if (!and_ok) {
 			warning(dict, "\"and\" and \"or\" at the same level in an expression");
 		}
 		if (!link_advance(dict)) {
 			return NULL;
 		}
-		nr = restricted_expression(dict, TRUE,FALSE);
+		nr = restricted_expression(dict, TRUE, FALSE);
 		if (nr == NULL) {
 			return NULL;
 		}
@@ -638,7 +930,12 @@ Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 		elr->e = nr;
 		n->type = AND_type;
 		n->cost = 0;
-	} else if (is_equal(dict, '|') || (strcmp(dict->token, "or")==0)) {
+		return n;
+	}
+	else if (is_equal(dict, '|') || (strcmp(dict->token, "or") == 0))
+	{
+		Exp *n;
+
 		if (!or_ok) {
 			warning(dict, "\"and\" and \"or\" at the same level in an expression");
 		}
@@ -658,53 +955,15 @@ Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 		elr->e = nr;
 		n->type = OR_type;
 		n->cost = 0;
-	} else return nl;
-	return n;
+		return n;
+	} 
+
+	return nl;
 }
 
 #endif
 
-/* The data structure storing the dictionary is simply a binary tree.  */
-/* There is one catch however.  The ordering of the words is not       */
-/* exactly the order given by strcmp.  It was necessary to             */
-/* modify the order to make it so that "make" < "make.n" < "make-up"   */
-/* The problem is that if some other string could  lie between '\0'    */
-/* and '.' (such as '-' which strcmp would give) then it makes it much */
-/* harder to return all the strings that match a given word.           */
-/* For example, if "make-up" was inserted, then "make" was inserted    */
-/* the a search was done for "make.n", the obvious algorithm would     */
-/* not find the match.                                                 */
-
-/* verbose version */
-/*
-int dict_compare(char *s, char *t)
-{
-	int ss, tt;
-	while (*s != '\0' && *s == *t) {
-		s++;
-		t++;
-	}
-	if (*s == '.') {
-		ss = 1;
-	} else {
-		ss = (*s)<<1;
-	}
-	if (*t == '.') {
-		tt = 1;
-	} else {
-		tt = (*t)<<1;
-	}
-	return (ss - tt);
-}
-*/
-
-/* terse version */
-static int dict_compare(const char *s, const char *t)
-{
-	while (*s != '\0' && *s == *t) {s++; t++;}
-	return (((*s == '.')?(1):((*s)<<1))  -  ((*t == '.')?(1):((*t)<<1)));
-}
-
+/* ======================================================================== */
 /**
  * Insert the new node into the dictionary below node n.
  * Give error message if the new element's string is already there.
@@ -712,22 +971,28 @@ static int dict_compare(const char *s, const char *t)
  * and right fields of it are NULL.
  *
  * XXX The resulting tree is rather highly unbalanced. It would improve
- * performance of rdictionary_lookup() by a factor of two if a tree
- * balancing step was run after the dictionary creation. XXX
+ * performance of rdictionary_lookup() by (at least!) a factor of two 
+ * if a tree balancing step was run after the dictionary creation! XXX
  */
 Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
 {
 	int comp;
-	char t[128];
 
 	if (n == NULL) return newnode;
-	comp = dict_compare(newnode->string, n->string);
-	if (comp < 0) {
+
+	comp = dict_order(newnode->string, n->string);
+	if (comp < 0)
+	{
 		n->left = insert_dict(dict, n->left, newnode);
-	} else if (comp > 0) {
+	}
+	else if (comp > 0)
+	{
 		n->right = insert_dict(dict, n->right, newnode);
-	} else {
-		sprintf(t, "The word \"%s\" has been multiply defined\n", newnode->string);
+	}
+	else
+	{
+		char t[256];
+		snprintf(t, 256, "The word \"%s\" has been multiply defined\n", newnode->string);
 		dict_error(dict, t);
 		return NULL;
 	}
@@ -740,6 +1005,11 @@ Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
  * l is the length of this list (the last ptr may not be NULL).
  * It inserts the list into the dictionary.
  * It does the middle one first, then the left half, then the right.
+ *
+ * XXX -- I think this insert middle, then left, then right, is a lame
+ * attempt to hack around the fact that the binary tree is rather
+ * badly unbalanced. Really, the correct thing to do would have been
+ * to just-plain balance the tree. See the other XXX above.
  */
 static void insert_list(Dictionary dict, Dict_node * p, int l)
 {
@@ -750,9 +1020,11 @@ static void insert_list(Dictionary dict, Dict_node * p, int l)
 
 	k = (l-1)/2;
 	dn = p;
-	for (i=0; i<k; i++) {
+	for (i = 0; i < k; i++)
+	{
 		dn = dn->left;
 	}
+
 	/* dn now points to the middle element */
 	dn_second_half = dn->left;
 	dn->left = dn->right = NULL;
@@ -760,16 +1032,16 @@ static void insert_list(Dictionary dict, Dict_node * p, int l)
 	if (contains_underbar(dn->string))
 	{
 		insert_idiom(dict, dn);
-	} 
+	}
 	else if (is_idiom_word(dn->string))
 	{
 		prt_error("Warning: Word \"%s\" found near line %d.\n"
 		        "\tWords ending \".Ix\" (x a number) are reserved for idioms.\n"
 		        "\tThis word will be ignored.\n",
 		        dn->string, dict->line_number);
-		xfree((char *)dn, sizeof(Dict_node));
+		free_dict_node(dn);
 	}
-	else if ((dn_head = abridged_lookup_list(dict, dn->string))!= NULL)
+	else if ((dn_head = abridged_lookup_list(dict, dn->string)) != NULL)
 	{
 		Dict_node *dnx;
 		prt_error("Warning: The word \"%s\" "
@@ -780,7 +1052,7 @@ static void insert_list(Dictionary dict, Dict_node * p, int l)
 		}
 		fprintf(stderr, "\n\tThis word will be ignored.\n");
 		free_lookup_list(dn_head);
-		xfree((char *)dn, sizeof(Dict_node));
+		free_dict_node(dn);
 	}
 	else
 	{
@@ -813,15 +1085,18 @@ static int read_entry(Dictionary dict)
 		}
 
 		/* if it's a word-file name */
-		if (dict->token[0] == '/') {
+		if (dict->token[0] == '/')
+		{
 			dn = read_word_file(dict, dn, dict->token);
 			if (dn == NULL)
 			{
 				prt_error("Error opening word file %s\n", dict->token);
 				return 0;
 			}
-		} else {
-			dn_new = (Dict_node *) xalloc(sizeof(Dict_node));
+		}
+		else
+		{
+			dn_new = dict_node_new();
 			dn_new->left = dn;
 			dn = dn_new;
 			dn->file = NULL;
@@ -829,35 +1104,35 @@ static int read_entry(Dictionary dict)
 		}
 
 		/* Advance to next entry, unless error */
-		if (0 == link_advance(dict)) return 0;
+		if (0 == link_advance(dict)) goto syntax_error;
 	}
 
 	/* pass the : */
 	if (!link_advance(dict))
 	{
-		return 0;
+		goto syntax_error;
 	}
 
 	n = expression(dict);
 	if (n == NULL)
 	{
-		return 0;
+		goto syntax_error;
 	}
 
 	if (!is_equal(dict, ';'))
 	{
 		dict_error(dict, "Expecting \";\" at the end of an entry.");
-		return 0;
+		goto syntax_error;
 	}
 
 	/* pass the ; */
 	if (!link_advance(dict))
 	{
-		return 0;
+		goto syntax_error;
 	}
 
-	/* at this point, dn points to a list of Dict_nodes connected by */
-	/* their left pointers. These are to be inserted into the dictionary */
+	/* At this point, dn points to a list of Dict_nodes connected by
+	 * their left pointers. These are to be inserted into the dictionary */
 	i = 0;
 	for (dnx = dn; dnx != NULL; dnx = dnx->left)
 	{
@@ -866,35 +1141,143 @@ static int read_entry(Dictionary dict)
 	}
 	insert_list(dict, dn, i);
 	return 1;
+
+syntax_error:
+	free_lookup_list(dn);
+	return 0;
 }
 
+#if ! defined INFIX_NOTATION
+/**
+ * print the expression, in prefix-style
+ */
 void print_expression(Exp * n)
 {
-	E_list * el; int i;
-	if (n == NULL) {
+	E_list * el;
+	int i;
+
+	if (n == NULL)
+	{
 		printf("NULL expression");
 		return;
 	}
-	if (n->type == CONNECTOR_type) {
+	if (n->type == CONNECTOR_type)
+	{
 		for (i=0; i<n->cost; i++) printf("[");
+		if (n->multi) printf("@");
 		printf("%s%c",n->u.string, n->dir);
-		for (i=0; i<n->cost; i++) printf("] ");
-	} else {
+		for (i=0; i<n->cost; i++) printf("]");
+		if (n->cost > 0) printf(" ");
+	}
+	else
+	{
 		for (i=0; i<n->cost; i++) printf("[");
 		if (n->cost == 0) printf("(");
 		if (n->type == AND_type) printf("& ");
 		if (n->type == OR_type) printf("or ");
-		for (el = n->u.l; el != NULL; el = el->next) {
+		for (el = n->u.l; el != NULL; el = el->next)
+		{
 			print_expression(el->e);
 		}
-		for (i=0; i<n->cost; i++) printf("] ");
+		for (i=0; i<n->cost; i++) printf("]");
+		if (n->cost > 0) printf(" ");
 		if (n->cost == 0) printf(") ");
 	}
 }
 
+#else /* INFIX_NOTATION */
+
+/**
+ * print the expression, in infix-style
+ */
+static void print_expression_parens(Exp * n, int need_parens)
+{
+	E_list * el;
+	int i;
+
+	if (n == NULL)
+	{
+		printf("NULL expression");
+		return;
+	}
+
+	/* print the connector only */
+	if (n->type == CONNECTOR_type)
+	{
+		for (i=0; i<n->cost; i++) printf("[");
+		if (n->multi) printf("@");
+		printf("%s%c",n->u.string, n->dir);
+		for (i=0; i<n->cost; i++) printf("]");
+		return;
+	}
+
+	/* Look for optional, and print only that */
+	el = n->u.l; 
+	if (el == NULL)
+	{
+		for (i=0; i<n->cost; i++) printf("[");
+		printf ("()");
+		for (i=0; i<n->cost; i++) printf("]");
+		return;
+	}
+
+	for (i=0; i<n->cost; i++) printf("[");
+	if ((n->type == OR_type) &&
+	    el && el->e && (NULL == el->e->u.l))
+	{
+		printf ("{");
+		print_expression_parens(el->next->e, FALSE);
+		printf ("}");
+		return;
+	}
+
+	if ((n->cost == 0) && need_parens) printf("(");
+
+	/* print left side of binary expr */
+	print_expression_parens(el->e, TRUE);
+
+	/* get a funny "and optional" when its a named expression thing. */
+	if ((n->type == AND_type) && (el->next == NULL))
+	{
+		return;
+	}
+
+	if (n->type == AND_type) printf(" & ");
+	if (n->type == OR_type) printf(" or ");
+
+	/* print right side of binary expr */
+	el = el->next;
+	if (el == NULL)
+	{
+		printf ("()");
+	}
+	else
+	{
+		if (el->e->type == n->type)
+		{
+			print_expression_parens(el->e, FALSE);
+		}
+		else
+		{
+			print_expression_parens(el->e, TRUE);
+		}
+		if (el->next != NULL)
+			printf ("\nERROR! Unexpected list!\n");
+	}
+
+	for (i=0; i<n->cost; i++) printf("]");
+	if ((n->cost == 0) && need_parens) printf(")");
+}
+
+void print_expression(Exp * n)
+{
+	print_expression_parens(n, FALSE);
+}
+#endif /* INFIX_NOTATION */
+
 static void rprint_dictionary_data(Dict_node * n)
 {
-		if (n==NULL) return;
+		if (n == NULL) return;
 		rprint_dictionary_data(n->left);
 		printf("%s: ", n->string);
 		print_expression(n->exp);
@@ -902,6 +1285,11 @@ static void rprint_dictionary_data(Dict_node * n)
 		rprint_dictionary_data(n->right);
 }
 
+/**
+ * Dump the entire contents of the dictionary
+ * XXX This is not currently called by anything, but is a "good thing
+ * to keep around".
+ */
 void print_dictionary_data(Dictionary dict)
 {
 	rprint_dictionary_data(dict->root);
@@ -923,187 +1311,7 @@ int read_dictionary(Dictionary dict)
 	return 1;
 }
 
-/**
- * dict_match() -- 
- * Assuming that s is a pointer to a dictionary string, and that
- * t is a pointer to a search string, this returns 0 if they
- * match, >0 if s>t, and <0 if s<t.
- *
- * The matching is done as follows.  Walk down the strings until
- * you come to the end of one of them, or until you find unequal
- * characters.  A "*" matches anything.  Otherwise, replace "."
- * by "\0", and take the difference.  This behavior matches that
- * of the function dict_compare().
- */
-static int dict_match(const char * s, const char * t)
-{
-	while((*s != '\0') && (*s == *t)) {s++; t++;}
-	if ((*s == '*') || (*t == '*')) return 0;
-	return (((*s == '.')?('\0'):(*s))  -  ((*t == '.')?('\0'):(*t)));
-}
-
-/**
- * We need to prune out the lists thus generated. 
- * A sub string will only be considered a subscript if it
- * followes the last "." in the word, and it does not begin
- * with a digit.
- */
-static int true_dict_match(const char * s, const char * t)
-{
-	char *ds, *dt;
-	ds = strrchr(s, '.');
-	dt = strrchr(t, '.');
-
-	/* a dot at the end or a dot followed by a number is NOT 
-	 * considered a subscript */
-	if ((dt != NULL) && ((*(dt+1) == '\0') || 
-	    (isdigit((int)*(dt+1))))) dt = NULL;
-	if ((ds != NULL) && ((*(ds+1) == '\0') ||
-	    (isdigit((int)*(ds+1))))) ds = NULL;
-
-	if (dt == NULL && ds != NULL) {
-		if (((int)strlen(t)) > (ds-s)) return FALSE;   /* we need to do this to ensure that */
-		return (strncmp(s, t, ds-s) == 0);	  /*"i.e." does not match "i.e" */
-	} else if (dt != NULL && ds == NULL) {
-		if (((int)strlen(s)) > (dt-t)) return FALSE;
-		return (strncmp(s, t, dt-t) == 0);
-	} else {
-		return (strcmp(s, t) == 0);
-	}
-}
-
 /* ======================================================================= */
-
-static Dict_node * prune_lookup_list(Dict_node *llist, const char * s)
-{
-	Dict_node *dn, *dnx, *dn_new;
-	dn_new = NULL;
-	for (dn = llist; dn!=NULL; dn = dnx) {
-		dnx = dn->right;
-		/* now put dn onto the answer list, or free it */
-		if (true_dict_match(dn->string, s)) {
-			dn->right = dn_new;
-			dn_new = dn;
-		} else {
-			xfree((char *)dn, sizeof(Dict_node));
-		}
-	}
-	/* now reverse the list back */
-	llist = NULL;
-	for (dn = dn_new; dn!=NULL; dn = dnx) {
-		dnx = dn->right;
-		dn->right = llist;
-		llist = dn;
-	}
-	return llist;
-}
-
-void free_lookup_list(Dict_node *llist)
-{
-	Dict_node * n;
-	while(llist != NULL) {
-		n = llist->right;
-		xfree((char *)llist, sizeof(Dict_node));
-		llist = n;
-	}
-}
-
-/**
- * rdictionary_lookup() --recursive dictionary lookup
- * Walks binary tree.
- */
-static Dict_node * rdictionary_lookup(Dict_node *llist, 
-                                      Dict_node * dn, const char * s)
-{
-	/* see comment in dictionary_lookup below */
-	int m;
-	Dict_node * dn_new;
-	if (dn == NULL) return llist;
-	m = dict_match(s, dn->string);
-	if (m >= 0) {
-		llist = rdictionary_lookup(llist, dn->right, s);
-	}
-	if (m == 0) {
-		dn_new = (Dict_node*) xalloc(sizeof(Dict_node));
-		*dn_new = *dn;
-		dn_new->right = llist;
-		llist = dn_new;
-	}
-	if (m <= 0) {
-		llist = rdictionary_lookup(llist, dn->left, s);
-	}
-	return llist;
-}
-
-/** 
- * dictionary_lookup_list() - return lookup list of words in the dictionary
- *
- * Returns a pointer to a lookup list of the words in the dictionary.
- * This list is made up of Dict_nodes, linked by their right pointers.
- * The node, file and string fields are copied from the dictionary.
- *
- * The returned list must be freed with free_lookup_list().
- */
-Dict_node * dictionary_lookup_list(Dictionary dict, const char *s)
-{
-   Dict_node * llist = rdictionary_lookup(NULL, dict->root, s);
-   llist = prune_lookup_list(llist, s);
-   return llist;
-}
-
-int boolean_dictionary_lookup(Dictionary dict, const char *s) 
-{
-	Dict_node *llist = dictionary_lookup_list(dict, s);
-	int bool = (llist != NULL);
-	free_lookup_list(llist);
-	return bool;
-}
-
-/**
- * The following routines are exactly the same as those above,
- * only they do not consider the idiom words
- */
-
-static Dict_node * rabridged_lookup(Dict_node *llist, Dict_node * dn, const char * s)
-{
-	int m;
-	Dict_node * dn_new;
-	if (dn == NULL) return llist;
-	m = dict_match(s, dn->string);
-	if (m >= 0) {
-		llist = rabridged_lookup(llist, dn->right, s);
-	}
-	if ((m == 0) && (!is_idiom_word(dn->string))) {
-		dn_new = (Dict_node*) xalloc(sizeof(Dict_node));
-		*dn_new = *dn;
-		dn_new->right = llist;
-		llist = dn_new;
-	}
-	if (m <= 0) {
-		llist = rabridged_lookup(llist, dn->left, s);
-	}
-	return llist;
-}
-
-static Dict_node * abridged_lookup_list(Dictionary dict, const char *s)
-{
-	Dict_node *llist;
-   llist = rabridged_lookup(NULL, dict->root, s);
-   llist = prune_lookup_list(llist, s);
-   return llist;
-}
-
-/**
- *  boolean_abridged_lookup() - returns true if in the dictionary, false otherwise
- */
-int boolean_abridged_lookup(Dictionary dict, const char *s)
-{
-	Dict_node *dn = abridged_lookup_list(dict, s);
-	int bool = (dn != NULL);
-	free_lookup_list(dn);
-	return bool;
-}
-
 /* the following functions are for handling deletion */
 /**
  * Returns true if it finds a non-idiom dict_node in a file that matches
@@ -1119,7 +1327,7 @@ static int find_one_non_idiom_node(Dict_node * p, Dict_node * dn,
 {
 	int m;
 	if (dn == NULL) return FALSE;
-	m = dict_match(s, dn->string);
+	m = dict_order_wild(s, dn->string);
 	if (m <= 0) {
 		if (find_one_non_idiom_node(dn,dn->left, s, parent, to_be_deleted)) return TRUE;
 	}
@@ -1156,6 +1364,9 @@ static void set_parent_of_node(Dictionary dict,
 /**
  * This deletes all the non-idiom words of the dictionary that match
  * the given string.  Returns TRUE if some deleted, FALSE otherwise.
+ *
+ * XXX Note: this function is not currently used anywhere in the code,
+ * but it could be useful for general dictionary editing.
  */
 int delete_dictionary_words(Dictionary dict, const char * s)
 {
@@ -1170,7 +1381,7 @@ int delete_dictionary_words(Dictionary dict, const char * s)
 		}
 		if (to_be_deleted->left == NULL) {
 			set_parent_of_node(dict, parent, to_be_deleted, to_be_deleted->right);
-			xfree((char *)to_be_deleted, sizeof(Dict_node));
+			free_dict_node(to_be_deleted);
 		} else {
 			pred_parent = to_be_deleted;
 			pred = to_be_deleted->left;
@@ -1182,7 +1393,7 @@ int delete_dictionary_words(Dictionary dict, const char * s)
 			to_be_deleted->file = pred->file;
 			to_be_deleted->exp = pred->exp;
 			set_parent_of_node(dict, pred_parent, pred, pred->left);
-			xfree((char *)pred, sizeof(Dict_node));
+			free_dict_node(pred);
 		}
 		if (!find_one_non_idiom_node(NULL, dict->root, s, &parent, &to_be_deleted)) return TRUE;
 	}
@@ -1216,26 +1427,20 @@ static void free_Elist(E_list * l)
 static void free_Exp_list(Exp * e)
 {
 	Exp * e1;
-	for (; e != NULL; e = e1) {
+	for (; e != NULL; e = e1)
+	{
 		e1 = e->next;
-		if (e->type != CONNECTOR_type) {
+		if (e->type != CONNECTOR_type)
+		{
 		   free_Elist(e->u.l);
 		}
-		xfree((char *)e, sizeof(Exp));
+		exp_free(e);
 	}
 }
 
-static void free_Dict_node(Dict_node * dn)
-{
-	if (dn == NULL) return;
-	free_Dict_node(dn->left);
-	free_Dict_node(dn->right);
-	xfree(dn, sizeof(Dict_node));
-}
-
 void free_dictionary(Dictionary dict)
 {
-	free_Dict_node(dict->root);
+	free_dict_node_recursive(dict->root);
 	free_Word_file(dict->word_file_header);
 	free_Exp_list(dict->exp_list);
 }
@@ -1243,7 +1448,7 @@ void free_dictionary(Dictionary dict)
 /**
  *  dict_display_word_info() - display the information about the given word.
  */
-void dict_display_word_info(Dictionary dict, const char * s) 
+void dict_display_word_info(Dictionary dict, const char * s)
 {
 	Dict_node *dn, *dn_head;
 	Disjunct * d1, * d2;
@@ -1257,16 +1462,19 @@ void dict_display_word_info(Dictionary dict, const char * s)
 	printf("Matches:\n");
 	for (dn = dn_head; dn != NULL; dn = dn->right)
 	{
-		len=0;
+		len = 0;
 		d1 = build_disjuncts_for_dict_node(dn);
-		for(d2 = d1 ; d2!=NULL; d2 = d2->next){
+		for(d2 = d1 ; d2 != NULL; d2 = d2->next)
+		{
 			len++;
 		}
 		free_disjuncts(d1);
-		printf("		  ");
-		left_print_string(stdout, dn->string, "				  ");
-		printf(" %5d  ", len);
-		if (dn->file != NULL) {
+		printf("    ");
+		left_print_string(stdout, dn->string,
+			"                         ");
+		printf(" %5d  disjuncts ", len);
+		if (dn->file != NULL)
+		{
 			printf("<%s>", dn->file->file);
 		}
 		printf("\n");
@@ -1274,3 +1482,29 @@ void dict_display_word_info(Dictionary dict, const char * s)
 	free_lookup_list(dn_head);
 	return;
 }
+
+/**
+ *  dict_display_word_expr() - display the connector info for a given word.
+ */
+void dict_display_word_expr(Dictionary dict, const char * s)
+{
+	Dict_node *dn, *dn_head;
+
+	dn_head = dictionary_lookup_list(dict, s);
+	if (dn_head == NULL)
+	{
+		printf("	\"%s\" matches nothing in the dictionary.\n", s);
+		return;
+	}
+	printf("\nExpressions:\n");
+	for (dn = dn_head; dn != NULL; dn = dn->right)
+	{
+		printf("    ");
+		left_print_string(stdout, dn->string,
+			"                         ");
+		print_expression(dn->exp);
+		printf("\n\n");
+	}
+	free_lookup_list(dn_head);
+	return;
+}
