@@ -202,8 +202,6 @@ struct Connector_struct
                      this is strictly a funcion of the connector
                      name, efficiency is the only reason to store
                      this.  If no limit, the value is set to 255. */
-  /*    unsigned char my_word; */  /* not used now */
-                  /* The word that this connector arises from */
     char priority;/* one of the three priorities above */
     char multi;   /* TRUE if this is a multi-connector */
     Connector * next;
@@ -242,15 +240,24 @@ struct Word_struct
     int firstupper;
 };
 
-/* The E_list an Exp structures defined below comprise the expression      */
-/* trees that are stored in the dictionary.  The expression has a type     */
-/* (AND, OR or TERMINAL).  If it is not a terminal it has a list           */
-/* (an E_list) of children.                                                */
+/** 
+ * Types of Exp_struct structures
+ */
+#define OR_type 0
+#define AND_type 1
+#define CONNECTOR_type 2
 
+/** 
+ * The E_list and Exp structures defined below comprise the expression
+ * trees that are stored in the dictionary.  The expression has a type
+ * (AND, OR or TERMINAL).  If it is not a terminal it has a list
+ * (an E_list) of children.
+ */
 typedef struct E_list_struct E_list;
 
-struct Exp_struct {
-    char type;            /* one of three types, see below */
+struct Exp_struct
+{
+    char type;            /* one of three types, see above */
     unsigned char cost;   /* The cost of using this expression.
                              Only used for non-terminals */
     char dir;   /* '-' means to the left, '+' means to right (for connector) */
@@ -258,20 +265,16 @@ struct Exp_struct {
     union {
         E_list * l;       /* only needed for non-terminals */
         char * string;    /* only needed if it's a connector */
-    }u;
+    } u;
     Exp * next;
 };
 
-struct E_list_struct {
+struct E_list_struct
+{
     E_list * next;
     Exp * e;
 };
 
-/* Here are the types */
-#define OR_type 0
-#define AND_type 1
-#define CONNECTOR_type 2
-
 /* The structure below stores a list of dictionary word files. */
 struct Word_file_struct
 {
@@ -280,10 +283,11 @@ struct Word_file_struct
     Word_file * next;
 };
 
-/* The dictionary is stored as a binary tree comprised of the following   */
-/* nodes.  A list of these (via right pointers) is used to return         */
-/* the result of a dictionary lookup.                                     */
-
+/** 
+ * The dictionary is stored as a binary tree comprised of the following
+ * nodes.  A list of these (via right pointers) is used to return
+ * the result of a dictionary lookup.
+ */
 struct Dict_node_struct
 {
     const char * string;  /* the word itself */
@@ -301,7 +305,8 @@ struct D_type_list_struct
 };
 
 typedef struct PP_node_struct PP_node;
-struct PP_node_struct {
+struct PP_node_struct
+{
     D_type_list *d_type_array[MAX_LINKS];
     char *violation;
 };
@@ -309,7 +314,8 @@ struct PP_node_struct {
 /* Davy added these */
 
 typedef struct Andlist_struct Andlist;
-struct Andlist_struct {
+struct Andlist_struct
+{
     Andlist * next;
     int conjunction;
     int num_elements;
@@ -322,7 +328,8 @@ struct Andlist_struct {
 /* This is for building the graphs of links in post-processing and          */
 /* fat link extraction.                                                     */
 
-struct Linkage_info_struct {
+struct Linkage_info_struct
+{
     int index;
     char fat;
     char canonical;
@@ -333,24 +340,24 @@ struct Linkage_info_struct {
     int island[MAX_SENTENCE];
 };
 
-struct List_o_links_struct{
+struct List_o_links_struct
+{
     int link;       /* the link number */
     int word;       /* the word at the other end of this link */
     int dir;        /* 0: undirected, 1: away from me, -1: toward me */
     List_o_links * next;
 };
 
+/* These parameters tell power_pruning, to tell whether this is before
+ * or after generating and disjuncts.  GENTLE is before RUTHLESS is 
+ * after.
+ */
 #define GENTLE 1
 #define RUTHLESS 0
-/* These parameters tell power_pruning, to tell whether this is before or after
-   generating and disjuncts.  GENTLE is before RUTHLESS is after. */
-
-/* int srandom(int);
-int random(void); */
-
 
 typedef struct string_node_struct String_node;
-struct string_node_struct {
+struct string_node_struct
+{
     char * string;
     int size;
     String_node * next;
@@ -390,7 +397,8 @@ struct X_table_connector_struct
 };
 
 /* from string-set.c */
-struct String_set_s {
+struct String_set_s
+{
     int size;       /* the current size of the table */
     int count;      /* number of things currently in the table */
     char ** table;  /* the table itself */
