@@ -18,6 +18,12 @@
 #include <string.h>
 #include <ctype.h>
 
+#ifdef USE_PTHREADS
+#include <pthread.h>
+#endif
+
+
+
 #ifdef ENABLE_BINRELOC
 #include "prefix.h"
 #endif /* BINRELOC */
@@ -32,6 +38,9 @@
 #endif
 
 #define IS_DIR_SEPARATOR(ch) (DIR_SEPARATOR == (ch))
+#ifdef _MSC_VER
+#define DICTIONARY_DIR "."
+#endif
 #define DEFAULTPATH DICTIONARY_DIR
 
 /* This file contains certain general utilities. */
@@ -140,13 +149,13 @@ void downcase_utf8_str(char *to, const char * from, size_t usize)
 
 	nbh = mbtowc (&c, from, 4);
 	c = towlower(c);
-	nbl = wctomb(low, c);
+	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
 	if ((nbh < nbl) && (to == from))
 	{
 		/* I'm to lazy to fix this */
-		fprintf(stderr, "Error: can't downcase multi-byte string!\n");
+		prt_error("Error: can't downcase multi-byte string!\n");
 		return;
 	}
 
@@ -171,13 +180,13 @@ void upcase_utf8_str(char *to, const char * from, size_t usize)
 
 	nbh = mbtowc (&c, from, 4);
 	c = towupper(c);
-	nbl = wctomb(low, c);
+	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
 	if ((nbh < nbl) && (to == from))
 	{
 		/* I'm to lazy to fix this */
-		fprintf(stderr, "Error: can't upcase multi-byte string!\n");
+		prt_error("Error: can't upcase multi-byte string!\n");
 		return;
 	}
 
@@ -192,67 +201,140 @@ void upcase_utf8_str(char *to, const char * from, size_t usize)
 }
 
 /* ============================================================= */
-/* memory alloc routines below */
+/* Memory alloc routines below. These routines attempt to keep
+ * track of how much space is getting used during a parse. 
+ *
+ * This code is probably obsolescent, and should probably be dumped.
+ */
 
-int max_space_in_use;
-int space_in_use;
-int max_external_space_in_use;
-int external_space_in_use;
+typedef struct
+{
+	size_t max_space_used;
+	size_t space_in_use;
+	size_t max_external_space_used;
+	size_t external_space_in_use;
+} space_t;
+
+#ifdef USE_PTHREADS
+static pthread_key_t space_key;
+static pthread_once_t space_key_once = PTHREAD_ONCE_INIT;
+
+static void space_key_alloc(void)
+{
+	pthread_key_create(&space_key, free);
+}
+#else
+static space_t space;
+#endif
+
+void init_memusage(void)
+{
+	space_t *s;
+
+	static int mem_inited = FALSE;
+	if (mem_inited) return;
+	mem_inited = TRUE;
+
+#ifdef USE_PTHREADS
+	pthread_once(&space_key_once, space_key_alloc);
+	s = (space_t *) malloc(sizeof(space_t));
+	pthread_setspecific(space_key, s);
+#else
+	s = &space;
+#endif
+
+	s->max_space_used = 0;
+	s->space_in_use = 0;
+	s->max_external_space_used = 0;
+	s->external_space_in_use = 0;
+}
+
+static inline space_t *getspace(void)
+{
+#ifdef USE_PTHREADS
+	return pthread_getspecific(space_key);
+#else
+	return &space;
+#endif
+}
 
 /**
- * To allow printing of a nice error message, and keep track of the
- *  space allocated.
+ * space used but not yet freed during parse
  */
-void * xalloc(int size)
+size_t get_space_in_use(void)
 {
-	char * p = (char *) malloc(size);
-	space_in_use += size;
-	if (space_in_use > max_space_in_use) max_space_in_use = space_in_use;
-	if ((p == NULL) && (size != 0)){
-		printf("Ran out of space.\n");
+	return getspace()->space_in_use;
+}
+
+/**
+ * maximum space used during the parse
+ */
+size_t get_max_space_used(void)
+{
+	return getspace()->max_space_used;
+}
+
+/**
+ * alloc some memory, and keep track of the space allocated.
+ */
+void * xalloc(size_t size)
+{
+	void * p = malloc(size);
+	space_t *s = getspace();
+	s->space_in_use += size;
+	if (s->max_space_used < s->space_in_use) s->max_space_used = s->space_in_use;
+	if ((p == NULL) && (size != 0))
+	{
+		prt_error("Fatal Error: Ran out of space.\n");
 		abort();
 		exit(1);
 	}
-	return (void *) p;
+	return p;
 }
 
-void xfree(void * p, int size)
+void * xrealloc(void *p, size_t oldsize, size_t newsize)
 {
-	space_in_use -= size;
-	free(p);
-}
-
-void * exalloc(int size)
-{
-	char * p = (char *) malloc(size);
-	external_space_in_use += size;
-	if (external_space_in_use > max_external_space_in_use) {
-		max_external_space_in_use = external_space_in_use;
-	}
-	if ((p == NULL) && (size != 0)){
-		printf("Ran out of space.\n");
+	space_t *s = getspace();
+	s->space_in_use -= oldsize;
+	p = realloc(p, newsize);
+	if ((p == NULL) && (newsize != 0))
+	{
+		prt_error("Fatal Error: Ran out of space on realloc.\n");
 		abort();
 		exit(1);
 	}
-	return (void *) p;
+	s->space_in_use += newsize;
+	if (s->max_space_used < s->space_in_use) s->max_space_used = s->space_in_use;
+	return p;
 }
 
-void exfree(void * p, int size) {
-	external_space_in_use -= size;
+void xfree(void * p, size_t size)
+{
+	getspace()->space_in_use -= size;
 	free(p);
 }
 
-/* This is provided as part of the API */
-void string_delete(char * p) {
-	exfree(p, strlen(p)+1);
+void * exalloc(size_t size)
+{
+	void * p = malloc(size);
+	space_t *s = getspace();
+	s->external_space_in_use += size;
+	if (s->max_external_space_used < s->external_space_in_use)
+		s->max_external_space_used = s->external_space_in_use;
+
+	if ((p == NULL) && (size != 0))
+	{
+		prt_error("Fatal Error: Ran out of space.\n");
+		abort();
+		exit(1);
+	}
+	return p;
 }
 
-/* Returns the smallest power of two that is at least i and at least 1 */
-int next_power_of_two_up(int i)
+void exfree(void * p, size_t size)
 {
-	int j=1;
-	while(j<i) j = j<<1;
-	return j;
+	getspace()->external_space_in_use -= size;
+	free(p);
 }
 
 /* =========================================================== */
@@ -366,7 +448,11 @@ path_get_dirname (const char *file_name)
 }
 #endif /* _WIN32 */
 
-char * custom_data_dir = NULL;
+/* global - but thats OK, since this is set only during initialization,
+ * and is is thenceforth a read-only item. So it doesn't need to be
+ * locked.
+ */
+static char * custom_data_dir = NULL;
 
 void set_data_dir(const char * path)
 {
@@ -390,12 +476,15 @@ static char * get_datadir(void)
 	data_dir = safe_strdup (BR_DATADIR("/link-grammar"));
 #elif defined(_WIN32)
 	/* Dynamically locate library and return containing directory */
-	hInstance = GetModuleHandle(NULL);
+	hInstance = GetModuleHandle("link-grammar.dll");
 	if(hInstance != NULL)
 	{
 		char dll_path[MAX_PATH];
 
 		if(GetModuleFileName(hInstance,dll_path,MAX_PATH)) {
+#ifdef _DEBUG
+			prt_error("Ifno: GetModuleFileName=%s\n", (dll_path?dll_path:"NULL"));
+#endif
 			data_dir = path_get_dirname(dll_path);
 		}
 	}
@@ -455,6 +544,9 @@ FILE *dictopen(const char *filename, const char *how)
 
 	{
 		char * data_dir = get_datadir();
+#ifdef _DEBUG
+		prt_error("Info: data_dir=%s\n", (data_dir?data_dir:"NULL"));
+#endif
 		if (data_dir) {
 			snprintf(fulldictpath, MAX_PATH_NAME, 
 			         "%s%c%s%c", data_dir, PATH_SEPARATOR, 
@@ -491,6 +583,9 @@ FILE *dictopen(const char *filename, const char *how)
 		strncpy(completename, oldpos, (pos-oldpos));
 		*(completename+(pos-oldpos)) = DIR_SEPARATOR;
 		strcpy(completename+(pos-oldpos)+1,filename);
+#ifdef _DEBUG
+		prt_error("Info: dictopen() trying %s\n", completename);
+#endif
 		if ((fp = fopen(completename, how)) != NULL) {
 			return fp;
 		}
@@ -500,8 +595,10 @@ FILE *dictopen(const char *filename, const char *how)
 }
 
 /* ======================================================== */
-/* Random number stuff below. Can this be replaced by 
- * standard system API's ??
+/*
+ * A static array of precomputed random values. This is used in a
+ * read-only fashion, and thus is thread-safe, despite being a global.
+ * Its initialized only once, during startup.
  */
 
 unsigned int randtable[RTSIZE];
@@ -516,48 +613,16 @@ unsigned int randtable[RTSIZE];
 void init_randtable(void)
 {
 	int i;
+	static int rand_table_inited = FALSE;
+	if (rand_table_inited) return;
+	rand_table_inited = TRUE;
+
 	srand(10);
 	for (i=0; i<RTSIZE; i++) {
 		randtable[i] = rand();
 	}
 }
 
-
-static int random_state[2] = {0,0};
-static int random_count = 0;
-static int random_inited = FALSE;
-
-static int step_generator(int d)
-{
-	/* no overflow should occur, so this is machine independent */
-	random_state[0] = ((random_state[0] * 3) + d + 104729) % 179424673;
-	random_state[1] = ((random_state[1] * 7) + d + 48611) % 86028121;
-	return random_state[0] + random_state[1];;
-}
-
-void my_random_initialize(int seed) 
-{
-	assert(!random_inited, "Random number generator not finalized.");
-
-	seed = (seed < 0) ? -seed : seed;
-	seed = seed % (1 << 30);
-
-	random_state[0] = seed % 3;
-	random_state[1] = seed % 5;
-	random_count = seed;
-	random_inited = TRUE;
-}
-
-void my_random_finalize(void) {
-	assert(random_inited, "Random number generator not initialized.");
-	random_inited = FALSE;
-}
-
-int my_random(void) {
-	random_count++;
-	return (step_generator(random_count));
-}
-
 /* ======================================================== */
 /* Locale routines */
 
