@@ -32,12 +32,15 @@ static const char * get_a_word(Dictionary dict, FILE * fp)
 
 	for (j=0; (j <= MAX_WORD-1) && (!iswspace(c)) && (c != WEOF);)
 	{
-		j += wctomb(&word[j], c);
+		j += wctomb_check(&word[j], c);
 		c = fgetwc(fp);
 	}
 
 	if (j >= MAX_WORD) {
-		error("The dictionary contains a word that is too long.");
+		word[MAX_WORD] = 0x0;
+		prt_error("Fatal Error: The dictionary contains a word that "
+		          "is too long. The word was: %s", word);
+		exit(1);
 	}
 	word[j] = '\0';
 	s = string_set_add(word, dict->string_set);
@@ -64,7 +67,7 @@ Dict_node * read_word_file(Dictionary dict, Dict_node * dn, char * filename)
 	safe_strcpy(file_name_copy, filename+1, sizeof(file_name_copy)); /* get rid of leading '/' */
 
 	if ((fp = dictopen(file_name_copy, "r")) == NULL) {
-		lperror(WORDFILE, "%s\n", file_name_copy);
+		prt_error("Error opening word file %s\n", file_name_copy);
 		return NULL;
 	}
 
@@ -88,9 +91,6 @@ Dict_node * read_word_file(Dictionary dict, Dict_node * dn, char * filename)
 	return dn;
 }
 
-#define LINE_LIMIT 70
-int xwhere_in_line;
-
 void save_files(Dictionary dict)
 {
 	Word_file *wf;
